@@ -25,12 +25,6 @@ and sequence state changes as output.
 Edits, undos and redos are done by creating and calling methods on these 
 EditAction objects and placing them on the undo/redo stack.
 """
-import copy
-import time
-import thread
-import sys
-import traceback
-
 import audiowaveform
 import appconsts
 import compositeeditor
@@ -39,8 +33,8 @@ from editorstate import get_track
 from editorstate import PLAYER
 import mltfilters
 import movemodes
-import projectdata
 import resync
+import trimmodes
 import undo
 import updater
 import utils
@@ -49,7 +43,7 @@ import utils
 do_gui_update = False
 # HACK. Used to hold references to all created compositors as some compositors 
 # crash program if garbage collected.
-old_compositors = []
+# old_compositors = []
 
 # ---------------------------------- atomic edit ops
 def append_clip(track, clip, clip_in, clip_out):
@@ -61,7 +55,7 @@ def append_clip(track, clip, clip_in, clip_out):
     track.clips.append(clip) # py
     track.append(clip, clip_in, clip_out) # mlt
     resync.clip_added_to_timeline(clip, track)
-    
+
 def _insert_clip(track, clip, index, clip_in, clip_out):
     """
     Affects MLT c-struct and python obj values.
@@ -150,14 +144,30 @@ def _frame_on_cut(clip, clip_frame):
         
     return False
 
-def _remove_trailing_blanks(track):
-    clip = track.get_clip(track.count() - 1)
-    try: # trying his on empty track fails 
-        while(clip.is_blank()):
-            _remove_clip(track, track.count() - 1)
-            clip = track.get_clip(track.count() - 1)
-    except:
-        pass
+def _remove_trailing_blanks_undo(self):
+    for trailing_blank in self.trailing_blanks:
+        track_index, length = trailing_blank 
+        track = current_sequence().tracks[track_index]
+        _insert_blank(track, track.count(), length)
+
+def _remove_trailing_blanks_redo(self):
+    _remove_all_trailing_blanks(self)
+
+def _remove_all_trailing_blanks(self=None):
+    if self != None:
+        self.trailing_blanks = []
+    for i in range(1, len(current_sequence().tracks) - 1): # -1 because hidden track, 1 because black track
+        try:
+            track = current_sequence().tracks[i]
+            last_clip_index = track.count() - 1
+            clip = track.clips[last_clip_index]
+            if clip.is_blanck_clip:
+                length = clip.clip_length()
+                _remove_clip(track, last_clip_index)
+                if self != None:
+                    self.trailing_blanks.append((i, length))
+        except:
+            pass
 
 def _create_clip_clone(clip):
     if clip.media_type != appconsts.PATTERN_PRODUCER:
@@ -167,15 +177,11 @@ def _create_clip_clone(clip):
     new_clip.name = clip.name
     return new_clip
 
-def _create_mute_volume_filter(seq):    
-    mute_filter = seq.create_filter(mltfilters.get_volume_filters_info())
-    mute_filter.mlt_filter.set("gain","0")
-    mute_filter.mlt_filter.set("end","0")
-    return mute_filter
-
+def _create_mute_volume_filter(seq): 
+    return mltfilters.create_mute_volume_filter(seq)
+    
 def _do_clip_mute(clip, volume_filter):
-    clip.attach(volume_filter.mlt_filter)
-    clip.mute_filter = volume_filter
+    mltfilters.do_clip_mute(clip, volume_filter)
 
 def _do_clip_unmute(clip):
     clip.detach(clip.mute_filter.mlt_filter)
@@ -200,12 +206,11 @@ def _overwrite_cut_track(track, frame, add_cloned_filters=False):
     and returns tuple of in and out frames of the clip that was cut as they
     were before the cut, for the purpose of having information to do undo later.
     
-    If cut was made it also clones fliters to new clip created by cut.
+    If cut was made it also clones fliters to new clip created by cut if requested.
     """
     index = track.get_clip_index_at(frame)
     clip = track.clips[index]
     orig_in_out = (clip.clip_in, clip.clip_out)
-    clip_out = clip.clip_out        
     clip_start_in_tline = track.clip_start(index)
     clip_frame = frame - clip_start_in_tline + clip.clip_in
     
@@ -263,10 +268,9 @@ def _overwrite_restore_out(track, moved_index, self):
             else: # blanks can't be resized, so put in new blank
                 _insert_blank(track, moved_index, self.out_clip_length)
             self.removed_clips.pop(-1) 
-    except Exception, err:
-        #print 'print_exc():'
-        #traceback.print_exc(file=sys.stdout)
-        pass
+    except:
+        pass # why?
+
 
 #---------------------------------------------- EDIT ACTION
 class EditAction:
@@ -288,13 +292,19 @@ class EditAction:
         # Grabs data as object members.
         self.__dict__.update(data)
         
+        # Other then actual trim edits, attempting all edits exits active trimodes and enters <X>_NO_EDIT trim mode.
+        self.exit_active_trimmode_on_edit = True
+        
         # HACK!!!! Overwrite edits crash at redo(sometimes undo) when current frame inside 
         # affected area if consumer running.
         # Remove when fixed upstream.
         self.stop_for_edit = False 
         self.turn_on_stop_for_edit = False # set true in redo_func for edits that need it
-        
+
     def do_edit(self):
+        if self.exit_active_trimmode_on_edit:
+            trimmodes.set_no_edit_trim_mode()
+
         self.redo()
         undo.register_edit(self)
         if self.turn_on_stop_for_edit:
@@ -308,9 +318,13 @@ class EditAction:
             PLAYER().consumer.stop()
 
         movemodes.clear_selected_clips()  # selection not valid after change in sequence
-
+        _remove_trailing_blanks_undo(self)
+        _consolidate_all_blanks_undo(self)
+    
         self.undo_func(self)
 
+        _remove_all_trailing_blanks(None)
+
         resync.calculate_and_set_child_clip_sync_states()
 
         # HACK, see above.
@@ -331,6 +345,8 @@ class EditAction:
 
         self.redo_func(self)
 
+        _consolidate_all_blanks_redo(self)
+        _remove_trailing_blanks_redo(self)
         resync.calculate_and_set_child_clip_sync_states()
 
         # HACK, see above.
@@ -340,12 +356,15 @@ class EditAction:
         if do_gui_update:
             self._update_gui()
 
-        
     def _update_gui(self):
         updater.update_tline_scrollbar() # Slider needs to adjust to possily new program length.
                                          # This REPAINTS TIMELINE as a side effect.
         updater.update_kf_editor()
 
+        current_sequence().update_edit_tracks_length() # NEEDED FOR TRIM CRASH HACK, REMOVE IF FIXED
+        current_sequence().update_trim_hack_blank_length()
+        PLAYER().display_inside_sequence_length(current_sequence().seq_len)
+
 
 # ---------------------------------------------------- SYNC DATA
 class SyncData:
@@ -461,8 +480,7 @@ def _insert_undo(self):
     _remove_clip(self.track, self.index)
 
 def _insert_redo(self):
-    _insert_clip(self.track, self.clip, self.index, self.clip_in, \
-                 self.clip_out)
+    _insert_clip(self.track, self.clip, self.index, self.clip_in, self.clip_out)
 
 
 #----------------- 3 POINT OVERWRITE
@@ -478,8 +496,7 @@ def _three_over_undo(self):
     for i in range(0, clips_count):
         add_clip = self.clips[i]
         index = self.in_index + i
-        _insert_clip(self.track, add_clip, index, add_clip.clip_in, \
-                     add_clip.clip_out)
+        _insert_clip(self.track, add_clip, index, add_clip.clip_in, add_clip.clip_out)
 
 def _three_over_redo(self):
     # Remove and replace
@@ -568,6 +585,7 @@ def _gap_append_redo(self):
 # "cut_frame"
 def tworoll_trim_action(data):
     action = EditAction(_tworoll_trim_undo,_tworoll_trim_redo, data)
+    action.exit_active_trimmode_on_edit = False
     return action
 
 def _tworoll_trim_undo(self):
@@ -590,8 +608,6 @@ def _tworoll_trim_undo(self):
         _insert_clip(self.track, self.to_clip, self.index, \
                      self.to_clip.clip_in - self.delta, \
                      self.to_clip.clip_out )
-
-    #self.edit_done_callback(False, self.cut_frame, self.delta, self.track, self.to_side_being_edited)
     
 def _tworoll_trim_redo(self):
     _remove_clip(self.track, self.index)
@@ -620,6 +636,29 @@ def _tworoll_trim_redo(self):
         self.first_do = False
         self.edit_done_callback(True, self.cut_frame, self.delta, self.track, self.to_side_being_edited)
 
+#----------------- SLIDE_TRIM
+# "track","clip","delta","index","first_do","first_do_callback","start_frame_being_viewed"
+def slide_trim_action(data):
+
+    action = EditAction(_slide_trim_undo,_slide_trim_redo, data)
+    action.exit_active_trimmode_on_edit = False
+    return action
+
+def _slide_trim_undo(self):
+    _remove_clip(self.track, self.index)
+    _insert_clip(self.track, self.clip, self.index,
+                 self.clip.clip_in - self.delta, self.clip.clip_out - self.delta)
+
+def _slide_trim_redo(self):
+    _remove_clip(self.track, self.index)
+    _insert_clip(self.track, self.clip, self.index,
+                 self.clip.clip_in + self.delta, self.clip.clip_out + self.delta)
+
+    # Reinit one roll trim 
+    if self.first_do == True:
+        self.first_do = False
+        self.first_do_callback(self.track, self.clip, self.index, self.start_frame_being_viewed)
+
 #-------------------- INSERT MOVE
 # "track","insert_index","selected_range_in","selected_range_out"
 # "move_edit_done_func"
@@ -664,6 +703,23 @@ def _insert_move_redo(self):
 
     self.move_edit_done_func(self.clips)
 
+# --------------------------------------- INSERT MULTIPLE
+# "track","clips","index"
+def insert_multiple_action(data):
+    action = EditAction(_insert_multiple_undo, _insert_multiple_redo, data)
+    return action
+
+def _insert_multiple_undo(self):
+    for i in range(0, len(self.clips)):
+        _remove_clip(self.track, self.index)
+
+def _insert_multiple_redo(self):
+    for i in range(0, len(self.clips)):
+        add_clip = self.clips[i]
+        index = self.index + i
+        _insert_clip(self.track, add_clip, index, add_clip.clip_in, add_clip.clip_out)
+
+
 #-------------------- MULTITRACK INSERT MOVE
 # "track","to_track","insert_index","selected_range_in","selected_range_out"
 # "move_edit_done_func"
@@ -701,9 +757,6 @@ def _multitrack_insert_move_redo(self):
         _insert_clip(self.to_track, clip, self.insert_index + i, \
                      clip.clip_in, clip.clip_out )
 
-    # Remove wrong sized waveforms
-    audiowaveform.maybe_delete_waveforms(self.clips, self.to_track)
-
     self.move_edit_done_func(self.clips)
     
 
@@ -751,7 +804,6 @@ def _overwrite_move_undo(self):
         clip = self.moved_clips[i];
         _insert_clip(track, clip, self.selected_range_in + i, clip.clip_in,
                      clip.clip_out)
-    _remove_trailing_blanks(track)
 
 def _overwrite_move_redo(self):
     self.moved_clips = []
@@ -794,8 +846,6 @@ def _overwrite_move_redo(self):
         clip = self.moved_clips[i]
         _insert_clip(track, clip, in_index + i, clip.clip_in, clip.clip_out)
 
-    _remove_trailing_blanks(track)
-
     # HACK, see EditAction for details
     self.turn_on_stop_for_edit = True
 
@@ -845,16 +895,13 @@ def _multitrack_overwrite_move_undo(self):
         clip = self.moved_clips[i];
         _insert_clip(track, clip, self.selected_range_in + i, clip.clip_in,
                      clip.clip_out)
-                     
-    _remove_trailing_blanks(track)
-    _remove_trailing_blanks(to_track)
 
 def _multitrack_overwrite_move_redo(self):
     self.moved_clips = []
     track = self.track
     to_track = self.to_track
 
-    # Lift moved clips and insert 
+    # Lift moved clips and insert blank
     for i in range(self.selected_range_in, self.selected_range_out + 1): # + 1 == out inclusive
         removed_clip = _remove_clip(track, self.selected_range_in) # THIS LINE BUGS SOMETIMES FIND OUT WHY
         self.moved_clips.append(removed_clip)
@@ -862,7 +909,7 @@ def _multitrack_overwrite_move_redo(self):
     removed_length = self.over_out - self.over_in
     _insert_blank(track, self.selected_range_in, removed_length)
 
-    # Find out if overwrite starts after track end and pad track with blanck if so
+    # Find out if overwrite starts after track end and pad track with blank if so
     if self.over_in >= to_track.get_length():
         self.starts_after_end = True
         gap = self.over_out - to_track.get_length()
@@ -890,32 +937,113 @@ def _multitrack_overwrite_move_redo(self):
     for i in range(0, len(self.moved_clips)):
         clip = self.moved_clips[i]
         _insert_clip(to_track, clip, in_index + i, clip.clip_in, clip.clip_out)
-
-    _remove_trailing_blanks(track)
-    _remove_trailing_blanks(to_track)
-
-    # Remove wrong sized waveforms
-    audiowaveform.maybe_delete_waveforms(self.moved_clips, to_track)
     
     # HACK, see EditAction for details
     self.turn_on_stop_for_edit = True
 
+#-------------------------------------------- MULTI MOVE
+# "multi_data", "edit_delta"
+# self.multi_data is multimovemode.MultimoveData
+def multi_move_action(data):
+    action = EditAction(_multi_move_undo, _multi_move_redo, data)
+    return action
+
+def _multi_move_undo(self):
+    track_moved = self.multi_data.track_affected    
+    tracks = current_sequence().tracks
+    for i in range(1, len(tracks) - 1):
+        if not track_moved[i - 1]:
+            continue
+        track = tracks[i]
+        edit_op = self.multi_data.track_edit_ops[i - 1]        
+        trim_blank_index = self.multi_data.trim_blank_indexes[i - 1]
+        
+        if edit_op == appconsts.MULTI_NOOP:
+            continue
+        elif edit_op == appconsts.MULTI_TRIM:
+            blank_length = track.clips[trim_blank_index].clip_length()
+            _remove_clip(track, trim_blank_index) 
+            _insert_blank(track, trim_blank_index, blank_length - self.edit_delta)
+        elif edit_op == appconsts.MULTI_ADD_TRIM:
+            _remove_clip(track, trim_blank_index) 
+        elif edit_op == appconsts.MULTI_TRIM_REMOVE:
+            if self.edit_delta != -self.multi_data.max_backwards:
+                _remove_clip(track, trim_blank_index) 
+                
+            _insert_blank(track, trim_blank_index, self.orig_length)
+
+    tracks_compositors = _get_tracks_compositors_list()
+    for i in range(1, len(tracks) - 1):
+        if not track_moved[i - 1]:
+            continue
+        track_comp = tracks_compositors[i - 1]
+        for comp in track_comp:
+            if comp.clip_in >= self.multi_data.first_moved_frame + self.edit_delta:
+                comp.move(-self.edit_delta)
+
+def _multi_move_redo(self):
+    tracks = current_sequence().tracks
+    track_moved = self.multi_data.track_affected
+
+    # Move clips          
+    for i in range(1, len(tracks) - 1):
+        if not track_moved[i - 1]:
+            continue
+        track = tracks[i]
+        edit_op = self.multi_data.track_edit_ops[i - 1]        
+        trim_blank_index = self.multi_data.trim_blank_indexes[i - 1]
+        
+        if edit_op == appconsts.MULTI_NOOP:
+            continue
+        elif edit_op == appconsts.MULTI_TRIM:
+            blank_length = track.clips[trim_blank_index].clip_length()
+            _remove_clip(track, trim_blank_index) 
+            _insert_blank(track, trim_blank_index, blank_length + self.edit_delta)
+        elif edit_op == appconsts.MULTI_ADD_TRIM:
+            _insert_blank(track, trim_blank_index, self.edit_delta)
+        elif edit_op == appconsts.MULTI_TRIM_REMOVE:
+            self.orig_length = track.clips[trim_blank_index].clip_length()
+            _remove_clip(track, trim_blank_index) 
+            if self.edit_delta != -self.multi_data.max_backwards:
+                _insert_blank(track, trim_blank_index, self.orig_length + self.edit_delta)
+
+    # Move compositors
+    tracks_compositors = _get_tracks_compositors_list()
+    for i in range(1, len(tracks) - 1):
+        if not track_moved[i - 1]:
+            continue
+        track_comp = tracks_compositors[i - 1]
+        for comp in track_comp:
+            if comp.clip_in >= self.multi_data.first_moved_frame:
+                comp.move(self.edit_delta)
+
+def _get_tracks_compositors_list():
+    tracks_list = []
+    tracks = current_sequence().tracks
+    compositors = current_sequence().compositors
+    for track_index in range(1, len(tracks) - 1):
+        track_compositors = []
+        for j in range(0, len(compositors)):
+            comp = compositors[j]
+            if comp.transition.b_track == track_index:
+                track_compositors.append(comp)
+        tracks_list.append(track_compositors)
     
+    return tracks_list
+
 #------------------ TRIM CLIP START
-# "track","clip","index","delta","undo_done_callback"
+# "track","clip","index","delta","first_do"
+# "undo_done_callback" <- THIS IS REALLY BADLY NAMED, IT SHOULD BE FIRST DO CALLBACK
 # Trims start of clip
 def trim_start_action(data): 
     action = EditAction(_trim_start_undo,_trim_start_redo, data)
+    action.exit_active_trimmode_on_edit = False
     return action
 
 def _trim_start_undo(self):
     _remove_clip(self.track, self.index)
     _insert_clip(self.track, self.clip, self.index,
                  self.clip.clip_in - self.delta, self.clip.clip_out)
-    
-    # Reinit one roll trim
-    #self.undo_done_callback(self.track, self.index, True)
-
 
 def _trim_start_redo(self):
     _remove_clip(self.track, self.index)
@@ -928,11 +1056,12 @@ def _trim_start_redo(self):
         self.undo_done_callback(self.track, self.index, True)
 
 #------------------ TRIM CLIP END
-# "track","clip","index","delta"
-# "undo_done_callback"
-# Trims start of clip
+# "track","clip","index","delta", "first_do"
+# "undo_done_callback" <- THIS IS REALLY BADLY NAMED, IT SHOULD BE FIRST DO CALLBACK
+# Trims end of clip
 def trim_end_action(data): 
     action = EditAction(_trim_end_undo,_trim_end_redo, data)
+    action.exit_active_trimmode_on_edit = False
     return action
 
 def _trim_end_undo(self):
@@ -940,9 +1069,6 @@ def _trim_end_undo(self):
     _insert_clip(self.track, self.clip, self.index,
                  self.clip.clip_in, self.clip.clip_out - self.delta)
     
-    # Reinit one roll trim
-    # self.undo_done_callback(self.track, self.index + 1, False)
-
 def _trim_end_redo(self):
     _remove_clip(self.track, self.index)
     _insert_clip(self.track, self.clip, self.index,
@@ -952,7 +1078,30 @@ def _trim_end_redo(self):
     if self.first_do == True:
         self.first_do = False
         self.undo_done_callback(self.track, self.index + 1, False)
-    
+
+#------------------ TRIM LAST CLIP END
+# "track","clip","index","delta", "first_do"
+# "undo_done_callback" <- THIS IS REALLY BADLY NAMED, IT SHOULD BE FIRST DO CALLBACK
+def trim_last_clip_end_action(data): 
+    action = EditAction(_trim_last_clip_end_undo,_trim_last_clip_end_redo, data)
+    action.exit_active_trimmode_on_edit = False
+    return action
+
+def _trim_last_clip_end_undo(self):
+    _remove_clip(self.track, self.index)
+    _insert_clip(self.track, self.clip, self.index,
+                 self.clip.clip_in, self.clip.clip_out - self.delta)
+
+def _trim_last_clip_end_redo(self):
+    _remove_clip(self.track, self.index)
+    _insert_clip(self.track, self.clip, self.index,
+                 self.clip.clip_in, self.clip.clip_out + self.delta)
+
+    # Reinit one roll trim for 
+    if self.first_do == True:
+        self.first_do = False
+        self.undo_done_callback(self.track)
+
 #------------------- ADD FILTER
 # "clip","filter_info","filter_edit_done_func"
 # Adds filter to clip.
@@ -1075,7 +1224,7 @@ def _clone_filters_redo(self):
     _attach_all(self.clip)
 
 # -------------------------------------- ADD COMPOSITOR ACTION
-# "origin_clip_id",in_frame","out_frame","compositor_index","a_track","b_track"
+# "origin_clip_id",in_frame","out_frame","compositor_type","a_track","b_track"
 def add_compositor_action(data):
     action = EditAction(_add_compositor_undo, _add_compositor_redo, data)
     action.first_do = True
@@ -1087,13 +1236,13 @@ def _add_compositor_undo(self):
     
     # Hack!!! Some filters don't seem to handle setting compositors None (and the
     # following gc) and crash, so we'll hold references to them forever.
-    global old_compositors
-    old_compositors.append(self.compositor)
+    #global old_compositors
+    #old_compositors.append(self.compositor)
     compositeeditor.maybe_clear_editor(self.compositor)
     self.compositor = None
 
 def _add_compositor_redo(self):    
-    self.compositor = current_sequence().create_compositor(self.compositor_index)
+    self.compositor = current_sequence().create_compositor(self.compositor_type)
     self.compositor.transition.set_tracks(self.a_track, self.b_track)
     self.compositor.set_in_and_out(self.in_frame, self.out_frame)
     self.compositor.origin_clip_id = self.origin_clip_id
@@ -1121,7 +1270,7 @@ def delete_compositor_action(data):
 def _delete_compositor_undo(self):
     old_compositor = self.compositor 
     
-    self.compositor = current_sequence().create_compositor(old_compositor.compositor_index)
+    self.compositor = current_sequence().create_compositor(old_compositor.type_id)
     self.compositor.clone_properties(old_compositor)
     self.compositor.set_in_and_out(old_compositor.clip_in, old_compositor.clip_out)
     self.compositor.transition.set_tracks(old_compositor.transition.a_track, old_compositor.transition.b_track)
@@ -1145,8 +1294,8 @@ def _delete_compositor_redo(self):
     
     # Hack!!! Some filters don't seem to handle setting compositors None (and the
     # following gc) and crash, so we'll hold references to them forever.
-    global old_compositors
-    old_compositors.append(self.compositor)
+    #global old_compositors
+    #old_compositors.append(self.compositor)
     
     compositeeditor.maybe_clear_editor(self.compositor)
 
@@ -1219,7 +1368,7 @@ def _audio_splice_undo(self):
 
     _do_clip_unmute(self.parent_clip)
     
-    _remove_trailing_blanks(to_track)
+    #_remove_trailing_blanks(to_track)
 
 def _audio_splice_redo(self):
     # Get shorter name for readability
@@ -1257,35 +1406,7 @@ def _audio_splice_redo(self):
 
     filter = _create_mute_volume_filter(current_sequence())
     _do_clip_mute(self.parent_clip, filter)
-    
-    _remove_trailing_blanks(to_track)
 
-# ------------------------------------------------- SPLIT TO SYNC
-# "parent_clip","audio_clip", "over_in","over_out","to_track","from_track","parent_index"
-# "to_track" == audio sync track
-# "from_track" == video master track
-def audio_sync_splice_action(data):
-    action = EditAction(_audio_sync_splice_undo, _audio_sync_splice_redo, data)
-    return action
-
-def _audio_sync_splice_undo(self):
-    _unmute_splice_parent_audio(self, self.parent_clip)
-
-def _audio_sync_splice_redo(self):
-    # Set sync data for audio clip
-    sync_data = SyncData()
-    sync_data.pos_offset = 0
-    sync_data.clip_in = self.parent_clip.clip_in
-    sync_data.clip_out = self.parent_clip.clip_out
-    sync_data.master_clip = self.parent_clip
-
-    _mute_splice_parent_audio(self, self.parent_clip)
-    sync_data.master_audio_index = self.parent_audio_index
-
-    self.audio_clip.sync_data = sync_data
-    #self.to_track.sync_clips.append(self.audio_clip)
-
-   
 # ------------------------------------------------- RESYNC ALL
 # No input data
 def resync_all_action(data):
@@ -1310,7 +1431,6 @@ def _resync_all_redo(self):
     resync_data = resync.get_resync_data_list()
     self.actions = _create_and_do_sync_actions_list(resync_data)
 
-
 # ------------------------------------------------- RESYNC SOME CLIPS
 # "clips"
 def resync_some_clips_action(data):
@@ -1363,6 +1483,44 @@ def _create_and_do_sync_actions_list(resync_data_list):
 
     return actions
 
+# ------------------------------------------------- RESYNC CLIP SEQUENCE
+# "clips"
+def resync_clips_sequence_action(data):
+    action = EditAction(_resync_clips_sequence_undo, _resync_clips_sequence_redo, data)
+    return action
+
+def _resync_clips_sequence_undo(self):
+     if self.sync_action != None:
+        self.sync_action.undo_func(self.sync_action)
+        
+def _resync_clips_sequence_redo(self):
+    resync_data = resync.get_resync_data_list_for_clip_list(self.clips)
+    clip, track, index, pos_offset = resync_data[0]
+
+    # If we're in sync, do nothing
+    if pos_offset == clip.sync_data.pos_offset:
+        self.sync_action = None
+    else:
+        # Get new in and out frames for clips 
+        diff = pos_offset - clip.sync_data.pos_offset
+        over_in = track.clip_start(index) - diff
+
+        clip_last, track, index_last, pos_offset = resync_data[-1]
+        last_over_in = track.clip_start(index_last) - diff
+        over_out = last_over_in + (clip_last.clip_out - clip_last.clip_in + 1)
+
+        # Create, do and sacve edit action.
+        data = {"track":track,
+                "over_in":over_in,
+                "over_out":over_out,
+                "selected_range_in":index,
+                "selected_range_out":index_last,
+                "move_edit_done_func":None}
+
+        action = overwrite_move_action(data)
+        action.redo_func(action)
+        self.sync_action = action
+
 # ------------------------------------------------- SET SYNC
 # "child_index","child_track","parent_index","parent_track"
 def set_sync_action(data):
@@ -1372,7 +1530,6 @@ def set_sync_action(data):
 def _set_sync_undo(self):
     # Get clips
     child_clip = self.child_track.clips[self.child_index]
-    parent_clip = self.parent_track.clips[self.parent_index]
      
     # Clear child sync data
     child_clip.sync_data = None
@@ -1414,10 +1571,10 @@ def _clear_sync_undo(self):
 def _clear_sync_redo(self):
     # Save sync data
     self.sync_data = self.child_clip.sync_data
-    
+
     # Clear child sync data
     self.child_clip.sync_data = None
-    
+
     # Claer resync data
     resync.clip_sync_cleared(self.child_clip)
     
@@ -1453,6 +1610,7 @@ def _unmute_clip_redo(self):
 #"track","clip","clip_index"
 def trim_end_over_blanks(data):
     action = EditAction(_trim_end_over_blanks_undo, _trim_end_over_blanks_redo, data)
+    action.exit_active_trimmode_on_edit = False
     return action 
 
 def _trim_end_over_blanks_undo(self):
@@ -1483,6 +1641,7 @@ def _trim_end_over_blanks_redo(self):
 # "track","clip","blank_index"
 def trim_start_over_blanks(data):
     action = EditAction(_trim_start_over_blanks_undo, _trim_start_over_blanks_redo, data)
+    action.exit_active_trimmode_on_edit = False
     return action
 
 def _trim_start_over_blanks_undo(self):
@@ -1543,7 +1702,7 @@ def _consolidate_all_blanks_undo(self):
         
 def _consolidate_all_blanks_redo(self):
     self.consolidate_actions = []
-    for i in range(1, len(current_sequence().tracks) - 1): # -1 because hidden track
+    for i in range(1, len(current_sequence().tracks) - 1): # -1 because hidden track, 1 because black track
         track = current_sequence().tracks[i]
         consolidaded_indexes = []
         try_do_next = True
@@ -1561,7 +1720,7 @@ def _consolidate_all_blanks_redo(self):
                     continue
                 except:
                     pass
-                
+
                 # Now consolidate from clip in index i
                 consolidaded_indexes.append(i)
                 removed_lengths = _remove_consecutive_blanks(track, i)
@@ -1599,9 +1758,108 @@ def _range_over_redo(self):
 
     # HACK, see EditAction for details
     self.turn_on_stop_for_edit = True
-                 
+
+
+#------------------- ADD CENTERED TRANSITION
+# "transition_clip","transition_index", "from_clip","to_clip","track","from_in","to_out"
+def add_centered_transition_action(data):
+    action = EditAction(_add_centered_transition_undo, _add_centered_transition_redo, data)
+    return action
+
+def _add_centered_transition_undo(self):
+    index = self.transition_index
+    track = self.track
+    from_clip = self.from_clip
+    to_clip = self.to_clip
+
+    for i in range(0, 3): # from, trans, to
+        _remove_clip(track, index - 1)
+    
+    _insert_clip(track, from_clip, index - 1, 
+                 from_clip.clip_in, self.orig_from_clip_out)
+    _insert_clip(track, to_clip, index, 
+                 self.orig_to_clip_in, to_clip.clip_out)
+
+def _add_centered_transition_redo(self):
+    # get shorter refs
+    transition_clip = self.transition_clip
+    index = self.transition_index
+    track = self.track
+    from_clip = self.from_clip
+    to_clip = self.to_clip
+    
+    # Save from and to clip in/out points before adding transition
+    self.orig_from_clip_out = from_clip.clip_out
+    self.orig_to_clip_in = to_clip.clip_in
+
+    # Shorten from clip    
+    _remove_clip(track, index - 1)
+    _insert_clip(track, from_clip, index - 1, 
+                 from_clip.clip_in, self.from_in) # self.from_in == transition start on from clip
+    # Shorten to clip 
+    _remove_clip(track, index)
+    _insert_clip(track, to_clip, index, 
+                         self.to_out + 1, to_clip.clip_out)  # self.to_out == transition end on to clip
+                                                             # + 1  == because frame is part of inserted transition
+    # Insert transition
+    _insert_clip(track, transition_clip, 
+                 self.transition_index, 1, # first frame is dropped as it is 100% from clip
+                 transition_clip.get_length() - 1)
+   
+
+# -------------------------------------------------------- RENDERED FADE IN
+# "fade_clip", "clip_index", "track", "length"
+def add_rendered_fade_in_action(data):
+    action = EditAction(_add_rendered_fade_in_undo, _add_rendered_fade_in_redo, data)
+    return action
+
+def _add_rendered_fade_in_undo(self):
+    _remove_clip(self.track, self.index)
+    _remove_clip(self.track, self.index)
+    _insert_clip(self.track,  self.orig_clip, self.index, self.orig_clip_in, self.orig_clip.clip_out)
+
+def _add_rendered_fade_in_redo(self):
+    self.orig_clip = _remove_clip(self.track, self.index)
+    self.orig_clip_in = self.orig_clip.clip_in 
+    _insert_clip(self.track, self.fade_clip, self.index, 0, self.length - 1)
+    _insert_clip(self.track,  self.orig_clip, self.index + 1, self.orig_clip.clip_in + self.length, self.orig_clip.clip_out)
+
+# -------------------------------------------------------- RENDERED FADE OUT
+# "fade_clip", "clip_index", "track", "length"
+def add_rendered_fade_out_action(data):
+    action = EditAction(_add_rendered_fade_out_undo, _add_rendered_fade_out_redo, data)
+    return action
+
+def _add_rendered_fade_out_undo(self):
+    _remove_clip(self.track, self.index)
+    _remove_clip(self.track, self.index)
+    _insert_clip(self.track,  self.orig_clip, self.index, self.orig_clip.clip_in, self.orig_clip_out)
+
+def _add_rendered_fade_out_redo(self):
+    self.orig_clip = _remove_clip(self.track, self.index)
+    self.orig_clip_out = self.orig_clip.clip_out 
+    _insert_clip(self.track,  self.orig_clip, self.index, self.orig_clip.clip_in, self.orig_clip.clip_out - self.length)
+    _insert_clip(self.track, self.fade_clip, self.index + 1, 0, self.length - 1)
+
+#-------------------- APPEND MEDIA LOG
+# "track","clips"
+def append_media_log_action(data):
+    action = EditAction(_append_media_log_undo,_append_media_log_redo, data)
+    return action
+
+def _append_media_log_undo(self):
+    for i in range(0, len(self.clips)):
+        _remove_clip(self.track, len(self.track.clips) - 1)
+    
+def _append_media_log_redo(self):
+    for i in range(0, len(self.clips)):
+        clip = self.clips[i]
+        append_clip(self.track, clip, clip.clip_in, clip.clip_out)
+
+
+
 # --------------------------------------------- help funcs for "range over" and "range splice out" edits
-# NOTE: RANGE SPLICE OUT NOT IMPLEMENTED YET; SO THIS BASICALLY UNNECESSARY METHOD.
+# NOTE: RANGE SPLICE OUT NOT IMPLEMENTED YET; SO THIS IS CURRENTLY DEAD CODE
 def _track_put_back_range(over_in, track, track_extract_data):
     # get index for first clip that was removed
     moved_index = track.get_clip_index_at(over_in)
@@ -1642,7 +1900,7 @@ def _track_put_back_range(over_in, track, track_extract_data):
         _insert_clip(track, clip, moved_index + i, clip.clip_in,
                      clip.clip_out)
                      
-    _remove_trailing_blanks(track)
+    #_remove_trailing_blanks(track)
 
 # NOTE: RANGE SPLICE OUT NOT IMPLEMENTED YET; SO THIS IS BASICALLY UNNECESSARY METHOD CAUSING 
 # CODE DUPLICATION WITH OTHER OVERWRITE METHODS
@@ -1698,7 +1956,7 @@ def _replace_with_speed_changed_clip_redo(self):
     # Create slowmo clip if it does not exists
     if not hasattr(self, "new_clip"):
         self.new_clip = current_sequence().create_slowmotion_producer(self.clip.path, self.speed)
-    current_sequence().clone_clip_range_and_filters(self.clip, self.new_clip)
+    current_sequence().clone_clip_and_filters(self.clip, self.new_clip)
     
     _remove_clip(self.track, self.clip_index)
     _insert_clip(self.track, self.new_clip, self.clip_index, self.clip.clip_in, self.clip.clip_out)
