@@ -28,19 +28,28 @@ import mlt
 import xml.dom.minidom
 
 import appconsts
+import editorstate
 from editorstate import PROJECT
+import mltrefhold
 import propertyparse
 import respaths
 import translations
 
 # Attr and node names in xml describing available filters.
 PROPERTY = appconsts.PROPERTY
+NON_MLT_PROPERTY = appconsts.NON_MLT_PROPERTY
 NAME = appconsts.NAME
 ARGS = appconsts.ARGS
 MLT_SERVICE = appconsts.MLT_SERVICE
+MLT_DROP_VERSION = "mlt_drop_version"
+MLT_MIN_VERSION = "mlt_min_version"
+EXTRA_EDITOR = appconsts.EXTRA_EDITOR
 FILTER = "filter"
 GROUP = "group"
 ID = "id"
+REPLACEMENT_RELATION = "replacementrelation"
+USE_SERVICE = "useservice"
+DROP_SERVICE = "dropservice"
 
 COMPOSITOR_FILTER_GROUP = "COMPOSITOR_FILTER" # THIS IS NOT USED ANYMORE! DOUBLE CHECK THAT THIS REALLY IS THE CASE AND KILL!
 MULTIPART_FILTER = "multipart" # identifies filter as multipart filter
@@ -66,8 +75,6 @@ group_icons = None
 # THIS IS NOT USED ANYMORE! DOUBLE CHECK THAT THIS REALLY IS THE CASE AND KILL!
 compositor_filters = {}
 
-
-
 # ICONS
 FILTER_DEFAULT_ICON = None
 
@@ -134,11 +141,20 @@ class FilterInfo:
             self.multipart_filter = (filter_node.getAttribute(MULTIPART_FILTER) == "true")
         except: # default is False
             self.multipart_filter = False
-             
+
+        try:
+            self.mlt_drop_version = filter_node.getAttribute(MLT_DROP_VERSION)
+        except: 
+            self.mlt_drop_version = None
+
+        try:
+            self.mlt_min_version = filter_node.getAttribute(MLT_MIN_VERSION)
+        except:
+            self.mlt_min_version = None
+
         self.xml = filter_node.toxml()
         self.name = filter_node.getElementsByTagName(NAME).item(0).firstChild.nodeValue
-        group_name = filter_node.getElementsByTagName(GROUP).item(0).firstChild # There's only one group node with text content child node
-        self.group = group_name.nodeValue
+        self.group = filter_node.getElementsByTagName(GROUP).item(0).firstChild.nodeValue
 
         # Properties saved as name-value-type tuplets
         p_node_list = filter_node.getElementsByTagName(PROPERTY)
@@ -159,10 +175,21 @@ class FilterInfo:
             end_property = mp.getAttribute(MULTIPART_END)
             self.multipart_desc = (args, start_property, end_property)
             self.multipart_value = value
-    
+
+        #  Extra editors that handle properties that have been set "no_editor"
+        e_node_list = filter_node.getElementsByTagName(EXTRA_EDITOR)
+        self.extra_editors = propertyparse.node_list_to_extraeditors_array(e_node_list)  
+
+        # Non-MLT properties are persistent values like properties that values are not directly written out as MLT properties
+        p_node_list = filter_node.getElementsByTagName(NON_MLT_PROPERTY)
+        self.non_mlt_properties = propertyparse.node_list_to_non_mlt_properties_array(p_node_list)
+        # Property args for Non-MLT properties saved in propertyname -> propertyargs_string dict
+        self.property_args.update(propertyparse.node_list_to_args_dict(p_node_list))
+        
     def get_icon(self):
         return _get_group_icon(self.group)
 
+
 class FilterObject:
     """
     These objects are saved with projects. Thay are used to generate, 
@@ -175,6 +202,7 @@ class FilterObject:
         self.info = filter_info
         # Values of these are edited by the user.
         self.properties = copy.deepcopy(filter_info.properties)
+        self.non_mlt_properties = copy.deepcopy(filter_info.non_mlt_properties) 
         self.mlt_filter = None # reference to MLT C-object
         self.active = True 
 
@@ -186,6 +214,7 @@ class FilterObject:
     
     def create_mlt_filter(self, mlt_profile):
         self.mlt_filter = mlt.Filter(mlt_profile, str(self.info.mlt_service_id))
+        mltrefhold.hold_ref(self.mlt_filter)
         self.update_mlt_filter_properties_all()
     
     def update_mlt_filter_properties_all(self):
@@ -203,7 +232,7 @@ class FilterObject:
         else:
              self.mlt_filter.set("disable", str(1))
     
-    def reset_values(self,  mlt_profile=None, clip=None): #multipartfilters need profile and clip and caller doesn't know difference
+    def reset_values(self,  mlt_profile=None, clip=None): #multipartfilters need profile and clip
         for i in range(0, len(self.properties)):
             name, o_value, prop_type = self.info.properties[i]
             name, value, prop_type = self.properties[i]
@@ -211,6 +240,7 @@ class FilterObject:
         
         self.update_mlt_filter_properties_all()
 
+
 class MultipartFilterObject:
     """
     These objects are saved with projects. Thay are used to generate, 
@@ -220,6 +250,7 @@ class MultipartFilterObject:
         self.info = filter_info
         # Values of these are edited by the user.
         self.properties = copy.deepcopy(filter_info.properties)
+        self.non_mlt_properties = copy.deepcopy(filter_info.non_mlt_properties) 
         self.value = copy.deepcopy(filter_info.multipart_value)
         self.active = True
         
@@ -259,6 +290,7 @@ class MultipartFilterObject:
     def create_filters_for_keyframes(self, keyframes, mlt_profile):
         for i in range(0, len(keyframes) - 1): # Theres one less filter parts than keyframes
             mlt_filter = mlt.Filter(mlt_profile, str(self.info.mlt_service_id))
+            mltrefhold.hold_ref(mlt_filter)
             self.mlt_filters.append(mlt_filter)
             
     def update_mlt_filters_values(self, keyframes):
@@ -338,6 +370,17 @@ def load_filters_xml(services):
     filter_nodes = filters_doc.getElementsByTagName(FILTER)
     for f_node in filter_nodes:
         filter_info = FilterInfo(f_node)
+        
+        if filter_info.mlt_drop_version != "":
+            if editorstate.mlt_version_is_equal_or_greater(filter_info.mlt_drop_version):
+                print filter_info.name + " dropped, MLT version too high for this filter."
+                continue
+
+        if filter_info.mlt_min_version != "":
+            if not editorstate.mlt_version_is_equal_or_greater(filter_info.mlt_min_version):
+                print filter_info.name + " dropped, MLT version too low for this filter."
+                continue
+
         if (not filter_info.mlt_service_id in services) and len(services) > 0:
             print "MLT service " + filter_info.mlt_service_id + " not found."
             global not_found_filters
@@ -347,6 +390,8 @@ def load_filters_xml(services):
         if filter_info.mlt_service_id == "volume": # we need this filter to do mutes so save reference to it
             global _volume_filter_info
             _volume_filter_info = filter_info
+
+        # Add filter compositor filters or filter groups
         if filter_info.group == COMPOSITOR_FILTER_GROUP:
             global compositor_filters
             compositor_filters[filter_info.name] = filter_info
@@ -378,18 +423,65 @@ def clone_filter_object(filter_object, mlt_profile):
     clone.create_mlt_filter(mlt_profile)
     return clone
 
+def replace_services(services):
+    # this has gotta be bullshit way to do this
+    
+    replacements_doc = xml.dom.minidom.parse(respaths.REPLACEMENTS_XML_DOC)
+
+    # Build dict that has enough info to enable deleting and finding filters by name
+    filters_dict = {}
+    for group_data in groups:
+        gkey, group = group_data
+        for f in group:
+            filters_dict[f.name] = (f, group)
+
+    # Replace services
+    replacement_nodes = replacements_doc.getElementsByTagName(REPLACEMENT_RELATION)
+    for r_node in replacement_nodes:
+
+        # Get use service values
+        use_node = r_node.getElementsByTagName(USE_SERVICE).item(0)
+        use_service_id = use_node.getAttribute(ID)
+        use_service_name = use_node.getAttribute(NAME)
+
+        # Try replace if use service and use filter exist
+        if (use_service_id in services) and len(services) > 0:
+            try:
+                use_service_data = filters_dict[use_service_name]
+            except:
+                print "Replace service " + use_service_name + " not found."
+                continue
+            
+            drop_nodes = r_node.getElementsByTagName(DROP_SERVICE)
+            
+            try:
+                # Drop service if found
+                for d_node in drop_nodes:
+                    drop_service_id = d_node.getAttribute(ID)
+                    drop_service_name = d_node.getAttribute(NAME)
+
+                    drop_service_data = filters_dict[drop_service_name]
+                    f_info, group = drop_service_data
+                    for i in range(0, len(group)):
+                        if group[i].name == f_info.name:
+                            group.pop(i)
+                            print f_info.name +" dropped for " + use_service_name
+                            break
+            except:
+                print "Dropping a mlt service for " + use_service_name + " failed, maybe not present."
+
 def get_compositor_filter(filter_id):
     return compositor_filters[filter_id]
 
-def get_audio_filters_group():
+def get_audio_filters_groups():
     for group_tuple in groups:
         gkey, group = group_tuple
         if gkey == translations.get_filter_group_name("Audio"):
-            return group
-    
-    # If we got here, something went wrong
-    print "no audio filters group found in mltfilters.get_audio_filters_group() !!!!!!!!!!!!!!!!!!!!"
-    return None
+            group_tuple1 = group_tuple
+        if gkey == translations.get_filter_group_name("Audio Filter"):
+            group_tuple2 = group_tuple
+
+    return [group_tuple1, group_tuple2]
 
 def get_volume_filters_info():
     return _volume_filter_info
@@ -414,3 +506,25 @@ def get_all_found_filters():
         gkey, group = group_tuple
         all_filters = all_filters + group
     return all_filters
+
+def print_found_filters():
+    all_filters = get_all_found_filters()
+    for f in all_filters:
+        print f.mlt_service_id + " for filter " + f.name  + " available"
+
+def print_not_found_filters():
+    for f in not_found_filters:
+        print f.mlt_service_id + " for filter " + f.name + " not found"
+
+
+# ------------------------------------------------------------- mute filters
+def create_mute_volume_filter(seq):    
+    mute_filter = seq.create_filter(get_volume_filters_info())
+    mute_filter.mlt_filter.set("gain","0")
+    mute_filter.mlt_filter.set("end","0")
+    return mute_filter
+
+def do_clip_mute(clip, volume_filter):
+    clip.attach(volume_filter.mlt_filter)
+    clip.mute_filter = volume_filter
+    
