@@ -23,17 +23,15 @@ Module contains objects that wrap mlt.Transition objects used to mix video betwe
 two tracks.
 """
 import copy
-import gtk
 import mlt
 import os
 import xml.dom.minidom
 
 import appconsts
-import mltfilters
-import persistance
+import mltrefhold
+import patternproducer
 import propertyparse
 import respaths
-import utils
 
 # Attr and node names in compositors.xml
 NAME = appconsts.NAME
@@ -48,23 +46,33 @@ PROP_INT = appconsts.PROP_INT
 PROP_FLOAT = appconsts.PROP_FLOAT
 PROP_EXPRESSION = appconsts.PROP_EXPRESSION
 
+# Renderered transitions
+RENDERED_DISSOLVE = appconsts.RENDERED_DISSOLVE
+RENDERED_WIPE = appconsts.RENDERED_WIPE
+RENDERED_COLOR_DIP = appconsts.RENDERED_COLOR_DIP
+RENDERED_FADE_IN = appconsts.RENDERED_FADE_IN
+RENDERED_FADE_OUT = appconsts.RENDERED_FADE_OUT
+
+rendered_transitions = None # list is set here at init_module() because otherwise translations can't be done (module load issue)
+
 # Info objects used to create mlt.Transitions for CompositorObject objects.
 # dict name : MLTCompositorInfo
 mlt_compositor_transition_infos = {}
 
+# Name -> type dict, used at creation when type is known, but name data has been left behind
+name_for_type = {}
+
 # Transitions not found in the system
 not_found_transitions = [] 
 
-# Wipes
-# User displayed name -> resource image
-wipe_lumas = None
+wipe_lumas = None # User displayed name -> resource image
 compositors = None
 blenders = None
 
 def init_module():
 
-    # translations and module load order make us do this
-    global wipe_lumas, compositors, blenders
+    # translations and module load order make us do this in method instead of at module load
+    global wipe_lumas, compositors, blenders, name_for_type, rendered_transitions, single_track_render_type_names
     wipe_lumas = { \
                 _("Vertical From Center"):"bi-linear_x.pgm",
                 _("Vertical Top to Bottom"):"wipe_top_to_bottom.svg",
@@ -111,13 +119,16 @@ def init_module():
                 _("Diagonal 4"):"wipe_diagonal_4.png",
                 _("Checkerboard"):"checkerboard_small.pgm"}
 
-    # Name -> creator funcion dict.
-    compositors = [ (_("Affine"),_create_affine_compositor),
-                    (_("Dissolve"),_create_dissolve_compositor),
-                    (_("Picture in Picture"),_create_pict_in_pict_compositor),
-                    (_("Region"), _create_region_wipe_compositor),
-                    #(_("Affine Blend"), _create_affine_blend_compositor),
-                    (_("Wipe Clip Length"), _create_wipe_compositor)]
+    # name -> mlt_compositor_transition_infos key dict.
+    unsorted_compositors = [ (_("Affine"),"##affine"),
+                             (_("Dissolve"),"##opacity_kf"),
+                             (_("Picture in Picture"),"##pict_in_pict"),
+                             (_("Region"), "##region"),
+                             (_("Affine Blend"), "##affineblend"),
+                             (_("Blend"), "##blend"),
+                             (_("Wipe Clip Length"),"##wipe")]
+
+    compositors = sorted(unsorted_compositors, key=lambda comp: comp[0])   
 
     # name -> mlt_compositor_transition_infos key dict.
     blenders = [(_("Add"),"##add"),
@@ -140,6 +151,21 @@ def init_module():
                 (_("Subtract"),"##subtract"),
                 (_("Value"),"##value")]
 
+    for comp in compositors:
+        name, comp_type = comp
+        name_for_type[comp_type] = name
+    
+    for blend in blenders:
+        name, comp_type = blend
+        name_for_type[comp_type] = name
+    
+    # change this, tuples are not need we only need list of translatd names
+    rendered_transitions = [  (_("Dissolve"), RENDERED_DISSOLVE), 
+                              (_("Wipe"), RENDERED_WIPE),
+                              (_("Color Dip"), RENDERED_COLOR_DIP),
+                              (_("Fade In"), RENDERED_FADE_IN),
+                              (_("Fade Out"), RENDERED_FADE_OUT)]
+
 # ------------------------------------------ compositors
 class CompositorTransitionInfo:
     """
@@ -158,7 +184,7 @@ class CompositorTransitionInfo:
         # Property args saved in propertyname -> propertyargs_string dict
         self.property_args = propertyparse.node_list_to_args_dict(p_node_list)
         
-        #  Extra editors that handle hardcoded properties that have been set "no_editor"
+        #  Extra editors that handle properties that have been set "no_editor"
         e_node_list = compositor_node.getElementsByTagName(EXTRA_EDITOR)
         self.extra_editors = propertyparse.node_list_to_extraeditors_array(e_node_list)  
 
@@ -179,9 +205,10 @@ class CompositorTransition:
         self.b_track = -1 # from, source
     
     def create_mlt_transition(self, mlt_profile):
-        self.mlt_transition = mlt.Transition(mlt_profile, 
-                                              str(self.info.mlt_service_id))
-                                              
+        transition = mlt.Transition(mlt_profile, 
+                                   str(self.info.mlt_service_id))
+        mltrefhold.hold_ref(transition)
+        self.mlt_transition = transition
         self.set_default_values()
         
         # PROP_EXPR values may have keywords that need to be replaced with
@@ -204,7 +231,6 @@ class CompositorTransition:
             self._set_blend_service_default_values()
         
     def _set_composite_service_default_values(self):
-        #self.mlt_transition.set("start", "0/0:100%x100%")
         self.mlt_transition.set("automatic",1)
         self.mlt_transition.set("aligned", 1)
         self.mlt_transition.set("deinterlace",0)
@@ -228,7 +254,7 @@ class CompositorTransition:
 
     def _set_region_service_default_values(self):
         self.mlt_transition.set("automatic",1)
-        self.mlt_transition.set("aligned", 1)
+        self.mlt_transition.set("aligned",1)
         self.mlt_transition.set("deinterlace",0)
         self.mlt_transition.set("distort",0)
         self.mlt_transition.set("fill",1)
@@ -280,7 +306,9 @@ class CompositorObject:
         self.name = None # ducktyping for clip for property editors
         self.selected = False
         self.origin_clip_id = None
-        self.destroy_id = os.urandom(16) # Objects are recreated often in Sequence.restack_compositors()
+        
+        self.destroy_id = os.urandom(16) # HACK, HACK, HACK - find a way to remove this stuff  
+                                         # Objects are recreated often in Sequence.restack_compositors()
                                          # and cannot be destroyd in undo/redo with object identidy.
                                          # This is cloned in clone_properties
 
@@ -288,6 +316,12 @@ class CompositorObject:
         # ducktyping for clip for property editors
         return self.clip_out - self.clip_in  + 1 # +1 out inclusive
 
+    def move(self, delta):
+        self.clip_in = self.clip_in + delta
+        self.clip_out = self.clip_out + delta
+        self.transition.mlt_transition.set("in", str(self.clip_in))
+        self.transition.mlt_transition.set("out", str(self.clip_out))
+
     def set_in_and_out(self, in_frame, out_frame):
         self.clip_in = in_frame
         self.clip_out = out_frame
@@ -322,59 +356,137 @@ def load_compositors_xml(transitions):
             continue
         mlt_compositor_transition_infos[compositor_info.name] = compositor_info
 
+def get_wipe_resource_path_for_sorted_keys_index(sorted_keys_index):
+    # This exists to avoid sending a list of sorted keys around or having to use global variables
+    keys = wipe_lumas.keys()
+    keys.sort()
+    return get_wipe_resource_path(keys[sorted_keys_index])
+    
 def get_wipe_resource_path(key):
     img_file = wipe_lumas[key]
     return respaths.WIPE_RESOURCES_PATH + img_file
 
-def create_compositor(compositor_type_index):
-    if compositor_type_index < len(compositors): #Create compositor
-        name, create_func = compositors[compositor_type_index]
-        compositor = create_func()
-    else: # Create blender compositor
-        name, transition_id_str = blenders[compositor_type_index - len(compositors)]
-        compositor = _create_blender(transition_id_str)
-    """
-    for persistance, used to recreate at load
-    INDEXES:
-    0                     - len(compositors) - 1                     --- compositors
-    len(compositors)      - len(compositors) + len(blenders) - 1     --- blenders
-    """
-    compositor.compositor_index = compositor_type_index
-    compositor.name = name
+def create_compositor(compositor_type):
+    transition_info = mlt_compositor_transition_infos[compositor_type]
+    compositor = CompositorObject(transition_info)
+    compositor.compositor_index = -1 # not used since SAVEFILE = 3
+    compositor.name = name_for_type[compositor_type]
+    compositor.type_id = compositor_type # this is a string like "##add", "##affineblend", in compositors.xml it is name element: <name>##affine</name> etc...
     return compositor
 
 
-# ------------------------------------------------- CompositorObject creators
-def _create_affine_compositor():
-    transition_info = mlt_compositor_transition_infos["##affine"]
-    return CompositorObject(transition_info)
+# ------------------------------------------------------ rendered transitions
+# These are tractor objects used to create rendered transitions.
+def get_rendered_transition_tractor(current_sequence, 
+                                    orig_from,
+                                    orig_to,
+                                    action_from_out,
+                                    action_from_in,
+                                    action_to_out,
+                                    action_to_in,
+                                    transition_type_selection_index,
+                                    wipe_luma_sorted_keys_index,
+                                    gdk_color_str):
 
-def _create_affine_blend_compositor():
-    transition_info = mlt_compositor_transition_infos["##affineblend"]
-    return CompositorObject(transition_info)
+    name, transition_type = rendered_transitions[transition_type_selection_index]
 
-def _create_pict_in_pict_compositor():
-    transition_info = mlt_compositor_transition_infos["##pict_in_pict"]
-    return CompositorObject(transition_info)
+    # New from clip
+    if orig_from.media_type != appconsts.PATTERN_PRODUCER:
+        from_clip = current_sequence.create_file_producer_clip(orig_from.path)# File producer
+    else:
+        from_clip = current_sequence.create_pattern_producer(orig_from.create_data) # pattern producer
+    current_sequence.clone_clip_and_filters(orig_from, from_clip)
 
-def _create_dissolve_compositor():
-    transition_info = mlt_compositor_transition_infos["##opacity_kf"]
-    return CompositorObject(transition_info)
+    # New to clip
+    if not(transition_type == RENDERED_FADE_IN or transition_type == RENDERED_FADE_OUT): # fades to not use to_clip
+        if orig_to.media_type != appconsts.PATTERN_PRODUCER:
+            to_clip = current_sequence.create_file_producer_clip(orig_to.path)# File producer
+        else:
+            to_clip = current_sequence.create_pattern_producer(orig_to.create_data) # pattern producer
+        current_sequence.clone_clip_and_filters(orig_to, to_clip)
 
-def _create_region_wipe_compositor():
-    transition_info = mlt_compositor_transition_infos["##region"]
-    return CompositorObject(transition_info)
+    # Create tractor and tracks
+    tractor = mlt.Tractor()
+    multitrack = tractor.multitrack()
+    track0 = mlt.Playlist()
+    track1 = mlt.Playlist()
+    multitrack.connect(track0, 0)
+    multitrack.connect(track1, 1)
 
-def _create_wipe_compositor():
-    transition_info = mlt_compositor_transition_infos["##wipe"]
-    return CompositorObject(transition_info)
+    # we'll set in and out points for images and pattern producers.
+    if not(transition_type == RENDERED_FADE_IN or transition_type == RENDERED_FADE_OUT): # fades to not use to_clip or some other data used here
+        if from_clip.media_type == appconsts.IMAGE or from_clip.media_type == appconsts.PATTERN_PRODUCER:
+            length = action_from_out - action_from_in
+            from_clip.clip_in = 0
+            from_clip.clip_out = length
 
-def _create_blender(transition_id_str):
-    transition_info = mlt_compositor_transition_infos[transition_id_str]
-    return CompositorObject(transition_info)
- 
-def _get_transition_wrapper_object(transition_info):
-    return CompositorTransition(transition_info)
+        if to_clip.media_type == appconsts.IMAGE or to_clip.media_type == appconsts.PATTERN_PRODUCER:
+            length = action_to_out - action_to_in
+            to_clip.clip_in = 0
+            to_clip.clip_out = length
+    else:
+        length = action_from_out
+        if from_clip.media_type == appconsts.IMAGE or from_clip.media_type == appconsts.PATTERN_PRODUCER:
+            from_clip.clip_in = 0
+            from_clip.clip_out = length
+            
+    # Add clips to tracks and create keyframe string to contron mixing
+    if transition_type == RENDERED_DISSOLVE or transition_type == RENDERED_WIPE:
+        # Add clips. Images and pattern producers always fill full track.
+        if from_clip.media_type != appconsts.IMAGE and from_clip.media_type != appconsts.PATTERN_PRODUCER:
+            track0.insert(from_clip, 0, action_from_in, action_from_out)
+        else:
+            track0.insert(from_clip, 0, 0, action_from_out - action_from_in)
+            
+        if to_clip.media_type != appconsts.IMAGE and to_clip.media_type != appconsts.PATTERN_PRODUCER: 
+            track1.insert(to_clip, 0, action_to_in, action_to_out)
+        else:
+            track1.insert(to_clip, 0, 0,  action_to_out - action_to_in)
+        kf_str = "0=0/0:100%x100%:0.0;"+ str(tractor.get_length() - 1) + "=0/0:100%x100%:100.0"
+    elif transition_type == RENDERED_COLOR_DIP:
+        length = action_from_out - action_from_in
+        first_clip_length = length / 2
+        second_clip_length = length - first_clip_length
+        color_clip = patternproducer.create_color_producer(current_sequence.profile, gdk_color_str)
+        track0.insert(color_clip, 0, 0, length)
+        track1.insert(from_clip, 0, action_from_in, action_from_in + first_clip_length)
+        track1.insert(to_clip, 1, action_to_out - second_clip_length, action_to_out)
+        kf_str = "0=0/0:100%x100%:100.0;"+ str(first_clip_length) + "=0/0:100%x100%:0.0;" + str(tractor.get_length() - 1) + "=0/0:100%x100%:100.0"
+    elif (transition_type == RENDERED_FADE_IN or transition_type == RENDERED_FADE_OUT):
+        color_clip = patternproducer.create_color_producer(current_sequence.profile, gdk_color_str)
+        track0.insert(color_clip, 0, 0, length)
+        if transition_type ==  RENDERED_FADE_IN:
+            track1.insert(from_clip, 0, orig_from.clip_in, orig_from.clip_in + length)
+            kf_str = "0=0/0:100%x100%:0.0;"+ str(length) + "=0/0:100%x100%:100.0"
+        else: # transition_type ==  RENDERED_FADE_OUT
+            track1.insert(from_clip, 0, orig_from.clip_out - length, orig_from.clip_out)
+            kf_str = "0=0/0:100%x100%:100.0;"+ str(length) + "=0/0:100%x100%:0.0"
 
+    # Create transition
+    transition = mlt.Transition(current_sequence.profile, "region")
+    mltrefhold.hold_ref(transition)
+    transition.set("composite.geometry", str(kf_str)) # controls mix over time
+    transition.set("composite.automatic",1)
+    transition.set("composite.aligned", 0)
+    transition.set("composite.deinterlace",0)
+    transition.set("composite.distort",0)
+    transition.set("composite.fill",1)
+    transition.set("composite.operator","over")
+    transition.set("composite.luma_invert",0)
+    transition.set("composite.progressive",1)
+    transition.set("composite.softness",0)
+    transition.set("in", 0)
+    transition.set("out", tractor.get_length() - 1)
+    transition.set("a_track", 0)
+    transition.set("b_track", 1)
 
+    # Setting luma resource file turns dissolve into wipe
+    if transition_type == RENDERED_WIPE:
+        wipe_resource_path = get_wipe_resource_path_for_sorted_keys_index(wipe_luma_sorted_keys_index)
+        transition.set("composite.luma", str(wipe_resource_path))
 
+    # Add transition
+    field = tractor.field()
+    field.plant_transition(transition, 0,1)
+
+    return tractor
