@@ -1,21 +1,21 @@
 """
-	Flowblade Movie Editor is a nonlinear video editor.
+    Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-	This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
 
-	Flowblade Movie Editor is free software: you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation, either version 3 of the License, or
-	(at your option) any later version.
+    Flowblade Movie Editor is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
 
-	Flowblade Movie Editor is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
+    Flowblade Movie Editor is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
+    You should have received a copy of the GNU General Public License
+    along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
 """
 
 """
@@ -24,9 +24,9 @@ Module handles user edit events for insert and over move modes.
 import gtk
 
 import appconsts
-import dialogs
+import dialogutils
+import dnd
 import edit
-import editorstate
 from editorstate import current_sequence
 from editorstate import get_track
 from editorstate import PLAYER
@@ -64,12 +64,10 @@ edit_data = None
 def play_pressed():
     # This handles only move modes, see trimmodes.py module for others.
     PLAYER().start_playback()  
-    updater.set_playing_configuration()
 
 def stop_pressed():
     # This handles only move modes, see trimmodes.py module for others.
     PLAYER().stop_playback() 
-    updater.set_stopped_configuration()
    
 def prev_pressed():
     # This handles only move modes, see trimmodes.py module for others.
@@ -92,6 +90,8 @@ def clear_selection_values():
     selected_track = -1
     selected_range_in = -1
     selected_range_out = -1
+    
+    updater.set_transition_render_edit_menu_items_sensitive(selected_range_in, selected_range_out)
 
 def set_range_selection(track_index, range_in, range_out, is_selected):
     """
@@ -100,6 +100,7 @@ def set_range_selection(track_index, range_in, range_out, is_selected):
     track = get_track(track_index)
     for i in range(range_in, range_out + 1): #+1, range_out is inclusive
         track.clips[i].selected = is_selected
+    
 
 def select_clip(track_index, clip_index):
     """
@@ -107,6 +108,7 @@ def select_clip(track_index, clip_index):
     """
     clear_selected_clips()
     set_range_selection(track_index, clip_index, clip_index, True)
+    updater.set_transition_render_edit_menu_items_sensitive(clip_index, clip_index)
     
     global selected_track, selected_range_in, selected_range_out
     selected_track = track_index
@@ -119,7 +121,8 @@ def _select_multiple_clips(track_index, range_start, range_end):
     """
     clear_selected_clips()
     set_range_selection(track_index, range_start, range_end, True)
-        
+    updater.set_transition_render_edit_menu_items_sensitive(range_start, range_end)
+
     global selected_track, selected_range_in, selected_range_out
     selected_track = track_index
     selected_range_in = range_start
@@ -347,8 +350,7 @@ def overwrite_move_release(x, y, frame, state):
     # clip is moved to a differrent track
     if track == to_track:
         # Do edit if were moving and clips have moved
-        if (edit_data["move_on"] == True
-            and(press_frame != frame)):
+        if (edit_data["move_on"] == True and (press_frame != frame)):
             clear_selected_clips()
             action = edit.overwrite_move_action(data)
             action.do_edit()
@@ -379,7 +381,6 @@ def _move_mode_pressed(event, frame):
     """
     x = event.x
     y = event.y
-    state = event.state
 
     global edit_data, pressed_on_selected, drag_disabled
 
@@ -455,15 +456,25 @@ def _move_mode_pressed(event, frame):
             pressed_on_selected = True
     # case: new single clip pressed
     else:
-        if not pressed_clip.is_blanck_clip:
+        if selected_track != track.id:
+            clear_selected_clips()
             select_clip(track.id, clip_index)
             pressed_on_selected = False
         else:
-            # Black clip, see comment aboce
-            range_in, range_out = _get_blanck_range(track, clip_index)
-            _select_multiple_clips(track.id, range_in, range_out)
-            pressed_on_selected = False
-            drag_disabled = True
+            if not pressed_clip.is_blanck_clip:
+                # Pressing on selected clip keeps selection unchanged
+                if clip_index < selected_range_in or clip_index > selected_range_out:
+                    select_clip(track.id, clip_index)
+                    pressed_on_selected = False
+                # Pressing on non-selected clip clears current selection and selects newly selected clip
+                else:
+                    pressed_on_selected = True
+            else:
+                # Black clip, see comment above
+                range_in, range_out = _get_blanck_range(track, clip_index)
+                _select_multiple_clips(track.id, range_in, range_out)
+                pressed_on_selected = False
+                drag_disabled = True
 
     # Get length info on selected clips
     clip_lengths = []
@@ -505,7 +516,6 @@ def _move_mode_move(frame, x, y):
     first_clip_start = edit_data["first_clip_start"]
     attempt_insert_frame = first_clip_start + (frame - press_frame)
     edit_data["attempt_insert_frame"] = attempt_insert_frame    
-    track = edit_data["track_object"]
     
     # Get track where insert is attempted. Track selection forced into range of editable tracks.
     to_track = tlinewidgets.get_track(y)
@@ -562,16 +572,30 @@ def _track_is_locked(track):
     global drag_disabled
     if track.edit_freedom == appconsts.LOCKED:
         track_name = utils.get_track_name(track, current_sequence())
-         
         # No edits on locked tracks.
-        # ???????????? text is shit
         primary_txt = _("Can't do edit on a locked track")
-        secondary_txt = _("Track ") + track_name + _(" is locked. Unlock track to edit it.\n") + \
-                        _("Deactive track ") + track_name + _(" if you wish to edit tracks below it.")
-        dialogs.warning_message(primary_txt, secondary_txt, gui.editor_window.window)
+        secondary_txt = _("Track ") + track_name + _(" is locked. Unlock track to edit it.\n")
+        dialogutils.warning_message(primary_txt, secondary_txt, gui.editor_window.window)
         
         drag_disabled = True
         return True
 
     return False
 
+# ------------------------------------- clip d'n'd to range log
+def clips_drag_out_started(event):
+    # Abort move edit
+    global edit_data, drag_disabled
+    edit_data = None
+    drag_disabled = True
+    tlinewidgets.set_edit_mode_data(None)
+    
+    # Set dnd
+    track = current_sequence().tracks[selected_track]
+    clips = []
+    for i in range(selected_range_in, selected_range_out + 1):
+        clips.append(track.clips[i])
+    dnd.start_tline_clips_out_drag(event, clips, gui.tline_canvas.widget)
+    
+    # Update tlione gui
+    updater.repaint_tline()
