@@ -32,11 +32,9 @@ import gtk
 
 import appconsts
 from editorstate import current_sequence
-import keyframeeditor
 import mlttransitions
 import mltfilters
 import propertyparse
-import respaths
 import utils
 
 # keys                                                      meaning of values for this key
@@ -60,12 +58,14 @@ OPACITY_IN_GEOM_SINGLE_KF = "opacity_in_geom_kf_single"     # 0=0/0:SCREEN_WIDTH
 OPACITY_IN_GEOM_KF = "opacity_in_geom_kf"                   # frame=0/0:SCREEN_WIDTHxSCREEN_HEIGHT:opacity (kf_str;kf_str;kf_str;...;kf_str)
 GEOMETRY_OPACITY_KF ="geom_opac_kf"                         # frame=x/y:widthxheight:opacity
 GEOM_IN_AFFINE_FILTER = "geom_in_affine_filt"               # x/y:widthxheight:opacity
+AFFINE_SCALE = "affine_scale"                               # special property to get the 1/ x that the filter wants
 KEYFRAME_HCS = "keyframe_hcs"                               # frame=value(;frame=value) HCS = half comma separeted
 KEYFRAME_HCS_TRANSITION = "keyframe_hcs_transition"         # frame=value(;frame=value) HCS = half comma separeted, used to edit transitions
 MULTIPART_KEYFRAME_HCS = "multipart_keyframe"               # frame=value(;frame=value) series of mlt.Filter objects that get their properties set, HCS = half comma separeted
 FREI_POSITION_HCS = "frei_pos_hcs"                          # frame=x:y
 FREI_GEOM_HCS_TRANSITION = "frei_geom_hcs";                 # time=x:y:x_scale:y_scale:rotation:mix
 COLOR = "color"                                             # #rrggbb
+LUT_TABLE = "lut_table"                                     # val;val;val;val;...;val
 WIPE_RESOURCE = "wipe_resource"                             # /path/to/resource.pgm
 NOT_PARSED = "not_parsed"                                   # A write out value is not parsed from value
 NOT_PARSED_TRANSITION = "not_parsed_transition"             # A write out value is not parsed from value in transition object
@@ -156,7 +156,17 @@ def get_transition_editable_properties(compositor):
     
     return editable_properties
 
-
+def get_non_mlt_editable_properties(clip, filter_object, filter_index):
+    editable_properties = []
+    for i in range(0, len(filter_object.non_mlt_properties)):
+        prop = filter_object.non_mlt_properties[i]
+        p_name, p_value, p_type = prop
+        args_str = filter_object.info.property_args[p_name]
+        ep = NonMltEditableProperty(prop, args_str, clip, filter_index, i)
+        editable_properties.append(ep)
+    
+    return editable_properties
+        
 # -------------------------------------------- property wrappers objs
 class AbstractProperty:
     """
@@ -339,7 +349,7 @@ class EditableProperty(AbstractProperty):
         self.write_mlt_property_str_value(str_value)
         self.value = str_value
         self.write_filter_object_property(str_value)
-
+        
     def write_mlt_property_str_value(self, str_value):
         # mlt property value
         filter_object = self._get_filter_object()
@@ -351,7 +361,7 @@ class EditableProperty(AbstractProperty):
         prop = (str(self.name), str(str_value), self.type)
         filter_object.properties[self.property_index] = prop
 
-      
+
 class TransitionEditableProperty(AbstractProperty):
     """
     A wrapper for mlttransitions.CompositorObject.transition.properties 
@@ -359,7 +369,7 @@ class TransitionEditableProperty(AbstractProperty):
     property values.
 
     This class is used for properties of type PROP_INT and PROP_FLOAT.
-    
+
     If property type is PROP_EXPRESSION an extending class is used
     to parse value expression from input.
     """
@@ -391,6 +401,40 @@ class TransitionEditableProperty(AbstractProperty):
         self.transition.properties[self.property_index] = prop
 
 
+class NonMltEditableProperty(AbstractProperty):
+    """
+    A wrapper for editable persistent properties that do not write out values to MLT objects.
+    Values of these are used to compute valuse that _are_ written to MLT.
+    """
+    def __init__(self, prop, args_str, clip, filter_index, non_mlt_property_index):
+        AbstractProperty.__init__(self, args_str)
+        self.name, self.value, self.type = prop
+        self.clip = clip
+        self.filter_index = filter_index
+        self.non_mlt_property_index = non_mlt_property_index
+        self.adjustment_listener = None # External listener that may be monkeypathched here
+
+    def adjustment_value_changed(self, adjustment):
+        if self.adjustment_listener != None:
+            value = adjustment.get_value()
+            out_value = self.get_out_value(value)
+            self.adjustment_listener(self, out_value)
+        
+    def _get_filter_object(self):
+        return self.clip.filters[self.filter_index]
+
+    def write_number_value(self, numb):
+        self.write_property_value(str(numb))
+
+    def write_property_value(self, str_value):
+        filter_object = self._get_filter_object()
+        prop = (str(self.name), str(str_value), self.type)
+        filter_object.non_mlt_properties[self.non_mlt_property_index] = prop
+        self.value = str_value
+
+    def get_float_value(self):
+        return float(self.value)
+
 # ----------------------------------------- PROP_EXPRESSION types extending classes
 class SingleKeyFrameProperty(EditableProperty):
     """
@@ -425,11 +469,12 @@ class AffineFilterGeomProperty(EditableProperty):
         x = x_s.get_adjustment().get_value()
         y = y_s.get_adjustment().get_value()
         h = h_s.get_adjustment().get_value()
-        
-        # x/y:widthxheight:opacity
-        val_str = str(x) + "/" + str(y) + ":" + str(w) + "x" + str(h) + ":100"
+
+        # "0=x/y:widthxheight:opacity"
+        val_str = "0=" + str(x) + "/" + str(y) + ":" + str(w) + "x" + str(h) + ":100" # 100x MLT ignores width
         self.write_value(val_str)
 
+
 class FreiPosHCSFilterProperty(EditableProperty):    
     def adjustment_value_changed(self, adjustment):
         value = adjustment.get_value()
@@ -502,6 +547,32 @@ class OpacityInGeomKeyframeProperty(TransitionEditableProperty):
         self.write_value(val_str)
 
 
+class LUTTableProperty(EditableProperty):
+    def reset_to_linear(self):
+        self.write_value("LINEAR")
+
+    def write_out_table(self, table):
+        l = []
+        for i in range(0, len(table)):
+
+            l.append(str(table[i]))
+            l.append(";")
+        val_str = ''.join(l).rstrip(";")
+        self.write_value(val_str)
+
+        
+class PointsListProperty(EditableProperty):
+    
+    def set_value_from_cr_points(self, crpoints):
+        val_str = ""
+        for i in range(0, len(crpoints)):
+            p = crpoints[i]
+            val_str = val_str + str(p.x) + "/"  + str(p.y)
+            if i < len(crpoints) - 1:
+                val_str = val_str + ";"
+        self.write_value(val_str)
+
+
 class KeyFrameGeometryOpacityProperty(TransitionEditableProperty):
     """
     Converts user edits to expressions like "12=11/21:720x576:76" for
@@ -533,10 +604,12 @@ class KeyFrameGeometryOpacityProperty(TransitionEditableProperty):
         val_str = val_str.strip(";")
         self.write_value(val_str)
 
+
 class FreiGeomHCSTransitionProperty(TransitionEditableProperty):
     def __init__(self, params):
         TransitionEditableProperty.__init__(self, params)
 
+
 class KeyFrameHCSFilterProperty(EditableProperty):
     """
     Coverts array of keyframe tuples to string of type "0=0.2;123=0.143"
@@ -558,6 +631,7 @@ class KeyFrameHCSFilterProperty(EditableProperty):
         val_str = val_str.strip(";")
         self.write_value(val_str)
 
+
 class KeyFrameHCSTransitionProperty(TransitionEditableProperty):
     """
     Coverts array of keyframe tuples to string of type "0=0.2;123=0.143"
@@ -572,16 +646,17 @@ class KeyFrameHCSTransitionProperty(TransitionEditableProperty):
             step = DEFAULT_STEP
         lower, upper = self.input_range
         return gtk.Adjustment(float(0.1), float(lower), float(upper), float(step)) # Value set later to first kf value
-        
+
     def write_out_keyframes(self, keyframes):
         val_str = ""
         for kf in keyframes:
             frame, val = kf
             val_str += str(frame) + "=" + str(self.get_out_value(val)) + ";"
-        
+
         val_str = val_str.strip(";")
         self.write_value(val_str)
 
+
 class ColorProperty(EditableProperty):
     """
     Gives value as gdk color for gui and writes out color as 
@@ -636,7 +711,7 @@ class MultipartKeyFrameProperty(AbstractProperty):
         except:
             step = DEFAULT_STEP
         lower, upper = self.input_range
-        ## stuff missing for load
+
         return gtk.Adjustment(float(0.1), float(lower), float(upper), float(step)) # Value set later to first kf value
 
     def write_out_keyframes(self, keyframes):
@@ -650,6 +725,52 @@ class MultipartKeyFrameProperty(AbstractProperty):
         filter_object.update_value(val_str, self.clip, current_sequence().profile)
 
 
+class AffineScaleProperty(EditableProperty):
+    
+    def get_input_range_adjustment(self):
+        step = DEFAULT_STEP
+        lower = 0
+        upper = 500
+        val = self.value.strip('"')
+        epxr_sides = val.split("=")
+        in_value = self.get_in_value(float(epxr_sides[1]))
+        return gtk.Adjustment(float(in_value), float(lower), float(upper), float(step))
+
+    def adjustment_value_changed(self, adjustment):
+        value = adjustment.get_value()
+        out_value = self.get_out_value(value)
+        val_str = "0=" + str(out_value)
+        self.write_value(val_str)
+
+    def get_out_value(self, in_value):
+        """
+        Converts input value to output value using ranges.
+        """
+        # in_range = 500 # hard coded special case
+        in_norm = in_value / 100.0 # to get 0 - 5, 1.0 no scaling
+        if in_norm < 0.001:
+            in_norm = 0.001
+        out =  1 / in_norm
+        return out
+
+    def get_in_value(self, out_value):
+        """
+        Converts output to input value
+        """
+        # out_value =  1 / in_norm, range 1 / 0.001 -> 1 / 5
+        if out_value < 0.001:
+            out_value = 0.001
+        in_value = (1 / (out_value)) * 100 # 0 - 500 range
+        return in_value  
+
+    def write_mlt_property_str_value(self, str_value):
+        val = str_value.strip('"')
+        epxr_sides = val.split("=")
+        # mlt property value
+        filter_object = self._get_filter_object()
+        filter_object.mlt_filter.set(str(self.name), str(epxr_sides[1]))
+        
+
 # ------------------------------------------ creator func dicts
 # dict EXPRESSION_TYPE args value -> class extending AbstractProperty
 # Note: HCS means half comma separated
@@ -667,7 +788,9 @@ EDITABLE_PROPERTY_CREATORS = { \
     COLOR: lambda params : ColorProperty(params),
     GEOMETRY_OPACITY_KF: lambda params : KeyFrameGeometryOpacityProperty(params),
     GEOM_IN_AFFINE_FILTER: lambda params : AffineFilterGeomProperty(params),
-    WIPE_RESOURCE : lambda(params) : WipeResourceProperty(params),
-    NOT_PARSED : lambda(params) : EditableProperty(params), # This should only be used with params that have editor=NO_EDITOR
-    NOT_PARSED_TRANSITION : lambda(params) : TransitionEditableProperty(params)   } # This should only be used with params that have editor=NO_EDITOR
+    WIPE_RESOURCE : lambda params : WipeResourceProperty(params),
+    LUT_TABLE : lambda params  : LUTTableProperty(params),
+    NOT_PARSED : lambda params : EditableProperty(params), # This should only be used with params that have editor=NO_EDITOR
+    NOT_PARSED_TRANSITION : lambda params : TransitionEditableProperty(params), # This should only be used with params that have editor=NO_EDITOR
+    AFFINE_SCALE : lambda params : AffineScaleProperty(params) }
 
