@@ -22,15 +22,18 @@
 Module creates GUI editors for editable mlt properties.
 """
 import gtk
-import sys
 
 import appconsts
 from editorstate import PROJECT
 from editorstate import PLAYER
+from editorstate import current_sequence
+import extraeditors
+import guiutils
 import keyframeeditor
 import mltfilters
 import mlttransitions
 import translations
+import utils
 
 EDITOR = "editor"
 
@@ -48,10 +51,14 @@ COMBO_BOX_OPTIONS = "cbopts"                                # List of options fo
 LADSPA_SLIDER = "ladspa_slider"                             # gtk.HScale, does ladspa update for release changes(disconnect, reconnect)
 CLIP_FRAME_SLIDER = "clip_frame_slider"                     # gtk.HScale, range 0 - clip length in frames
 AFFINE_GEOM_4_SLIDER = "affine_filt_geom_slider"            # 4 rows of gtk.HScales to set the position and size
+COLOR_CORRECTOR = "color_corrector"                         # 3 band color corrector color circle and Lift Gain Gamma sliders
+CR_CURVES = "crcurves"                                      # Curves color editor with Catmull-Rom curve
+COLOR_BOX = "colorbox"                                      # One band color editor with color box interface
 NO_EDITOR = "no_editor"                                     # No editor displayed for property
 
 COMPOSITE_EDITOR_BUILDER = "composite_properties"           # Creates a single row editor for multiple properties of composite transition
 REGION_EDITOR_BUILDER = "region_properties"                 # Creates a single row editor for multiple properties of region transition
+ROTATION_GEOMETRY_EDITOR_BUILDER = "rotation_geometry_editor" # Creates a single editor for multiple geometry values
 
 SCALE_DIGITS = "scale_digits"                               # Number of decimal digits displayed in a widget
 
@@ -61,7 +68,7 @@ def _p(name):
     except KeyError:
         return name
 
-# ------------------------------------------------- gui components
+
 def get_editor_row(editable_property):
     """
     Returns GUI component to edit provided editable property.
@@ -90,6 +97,24 @@ def get_transition_extra_editor_rows(compositor, editable_properties):
 
     return rows
 
+def get_filter_extra_editor_rows(filt, editable_properties):
+    """
+    Returns list of extraeditors GUI components.
+    """
+    #print editable_properties
+    extra_editors = filt.info.extra_editors
+    rows = []
+    for editor_name in extra_editors:
+        try:
+            create_func = EDITOR_ROW_CREATORS[editor_name]
+            editor_row = create_func(filt, editable_properties)
+            rows.append(editor_row)
+        except KeyError:
+            print "get_transition_extra_editor_rows fail with:" + editor_name
+
+    return rows
+    
+# ------------------------------------------------- gui builders
 def _get_two_column_editor_row(name, editor_widget):
     name = _p(name)
     label = gtk.Label(name + ":")
@@ -104,7 +129,7 @@ def _get_two_column_editor_row(name, editor_widget):
     hbox.pack_start(editor_widget, True, True, 0)
     return hbox
     
-def _get_slider_row(editable_property, slider_name=None):
+def _get_slider_row(editable_property, slider_name=None, compact=False):
     adjustment = editable_property.get_input_range_adjustment()
     adjustment.connect("value-changed", editable_property.adjustment_value_changed)
 
@@ -113,22 +138,31 @@ def _get_slider_row(editable_property, slider_name=None):
     hslider.set_draw_value(False)
 
     spin = gtk.SpinButton()
+    spin.set_numeric(True)
     spin.set_adjustment(adjustment)
 
     _set_digits(editable_property, hslider, spin)
 
-    hbox = gtk.HBox(False, 4)
-    hbox.pack_start(hslider, True, True, 0)
-    hbox.pack_start(spin, False, False, 4)
     if slider_name == None:
         name = editable_property.get_display_name()
     else:
         name = slider_name
     name = _p(name)
-    top_row = _get_two_column_editor_row(name, gtk.HBox())
+    
+    hbox = gtk.HBox(False, 4)
+    if compact:
+        name_label = gtk.Label(name + ":")
+        hbox.pack_start(name_label, False, False, 4)
+    hbox.pack_start(hslider, True, True, 0)
+    hbox.pack_start(spin, False, False, 4)
+
     vbox = gtk.VBox(False)
-    vbox.pack_start(top_row, True, True, 0)
-    vbox.pack_start(hbox, False, False, 0)
+    if compact:
+        vbox.pack_start(hbox, False, False, 0)
+    else:
+        top_row = _get_two_column_editor_row(name, gtk.HBox())
+        vbox.pack_start(top_row, True, True, 0)
+        vbox.pack_start(hbox, False, False, 0)
     return vbox
 
 def _get_ladspa_slider_row(editable_property, slider_name=None):
@@ -140,6 +174,7 @@ def _get_ladspa_slider_row(editable_property, slider_name=None):
     hslider.connect("button-release-event", lambda w, e: _ladspa_slider_update(editable_property, adjustment))
     
     spin = gtk.SpinButton()
+    spin.set_numeric(True)
     spin.set_adjustment(adjustment)
     spin.connect("button-release-event", lambda w, e: _ladspa_slider_update(editable_property, adjustment))
 
@@ -152,7 +187,7 @@ def _get_ladspa_slider_row(editable_property, slider_name=None):
         name = editable_property.get_display_name()
     else:
         name = slider_name
-    #return _get_two_column_editor_row(name, hbox)
+
     top_row = _get_two_column_editor_row(name, gtk.HBox())
     vbox = gtk.VBox(False)
     vbox.pack_start(top_row, True, True, 0)
@@ -176,6 +211,7 @@ def _get_clip_frame_slider(editable_property):
     hslider.connect("button-release-event", lambda w, e: _clip_frame_slider_update(editable_property, adjustment))
     
     spin = gtk.SpinButton()
+    spin.set_numeric(True)
     spin.set_adjustment(adjustment)
     spin.connect("button-release-event", lambda w, e: _clip_frame_slider_update(editable_property, adjustment))
 
@@ -192,19 +228,20 @@ def _get_clip_frame_slider(editable_property):
 def _get_affine_filt_geom_sliders(ep):
     scr_width = PROJECT().profile.width()
     scr_height = PROJECT().profile.width()
-    
-    #0,0:SCREENSIZE:100
-    tokens = ep.value.split(":")
+
+    # "0=0,0:SCREENSIZE:100"
+    frame_value = ep.value.split("=")
+    tokens = frame_value[1].split(":")
     pos_tokens = tokens[0].split("/")
     size_tokens = tokens[1].split("x")
 
     x_adj = gtk.Adjustment(float(pos_tokens[0]), float(-scr_width), float(scr_width), float(1))
     y_adj = gtk.Adjustment(float(pos_tokens[1]), float(-scr_height), float(scr_height), float(1))
-    h_adj = gtk.Adjustment(float(size_tokens[1]), float(0), float(scr_height * 2), float(1))
-
+    h_adj = gtk.Adjustment(float(size_tokens[1]), float(0), float(scr_height * 5), float(1))
+    
     x_slider, x_spin, x_row =  _get_affine_slider("X", x_adj)
     y_slider, y_spin, y_row =  _get_affine_slider("Y", y_adj)
-    h_slider, h_spin, h_row =  _get_affine_slider(_("Height"), h_adj)
+    h_slider, h_spin, h_row =  _get_affine_slider(_("Size/Height"), h_adj)
 
     all_sliders = (x_slider, y_slider, h_slider)
 
@@ -215,6 +252,7 @@ def _get_affine_filt_geom_sliders(ep):
     h_slider.get_adjustment().connect("value-changed", lambda w: ep.slider_values_changed(all_sliders, scr_width))
     h_spin.get_adjustment().connect("value-changed", lambda w: ep.slider_values_changed(all_sliders, scr_width))
 
+    
     vbox = gtk.VBox(False, 4)
     vbox.pack_start(x_row, True, True, 0)
     vbox.pack_start(y_row, True, True, 0)
@@ -228,6 +266,7 @@ def _get_affine_slider(name, adjustment):
     hslider.set_draw_value(False)
     
     spin = gtk.SpinButton()
+    spin.set_numeric(True)
     spin.set_adjustment(adjustment)
 
     hslider.set_digits(0)
@@ -289,6 +328,7 @@ def _get_wipe_selector(editable_property):
     """
     Returns GUI component for selecting wipe type.
     """
+    # Preset luma
     combo_box = gtk.combo_box_new_text()
             
     # Get options
@@ -307,20 +347,87 @@ def _get_wipe_selector(editable_property):
             k_index = keys.index(k)
     
     combo_box.set_active(k_index)
+    preset_luma_row = _get_two_column_editor_row(editable_property.get_display_name(), combo_box)
+    
+    # User luma
+    use_preset_luma_combo = gtk.combo_box_new_text()
+    use_preset_luma_combo.append_text(_("Preset Luma"))
+    use_preset_luma_combo.append_text(_("User Luma"))
+        
+    dialog = gtk.FileChooserDialog(_("Select Luma File"), None, 
+                                   gtk.FILE_CHOOSER_ACTION_OPEN, 
+                                   (_("Cancel").encode('utf-8'), gtk.RESPONSE_REJECT,
+                                    _("OK").encode('utf-8'), gtk.RESPONSE_ACCEPT), None)
+    dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
+    dialog.set_select_multiple(False)
+    file_filter = gtk.FileFilter()
+    file_filter.add_pattern("*.png")
+    file_filter.add_pattern("*.pgm")
+    file_filter.set_name(_("Wipe Luma files"))
+    dialog.add_filter(file_filter)
+        
+    user_luma_select = gtk.FileChooserButton(dialog)
+    user_luma_select.set_size_request(210, 28)
+    
+    user_luma_label = gtk.Label(_("Luma File:"))
+
+    if k_index == -1:
+        use_preset_luma_combo.set_active(1)
+        combo_box.set_sensitive(False)
+        combo_box.set_active(0)
+        user_luma_select.set_filename(editable_property.value)
+    else:
+        use_preset_luma_combo.set_active(0)
+        user_luma_select.set_sensitive(False)
+        user_luma_label.set_sensitive(False)
+    
+    user_luma_row = gtk.HBox(False, 2)
+    user_luma_row.pack_start(use_preset_luma_combo, False, False, 0)
+    user_luma_row.pack_start(gtk.Label(), True, True, 0)
+    user_luma_row.pack_start(user_luma_label, False, False, 2)
+    user_luma_row.pack_start(user_luma_select, False, False, 0)
+
+    editor_pane = gtk.VBox(False)
+    editor_pane.pack_start(preset_luma_row, False, False, 4)
+    editor_pane.pack_start(user_luma_row, False, False, 4)
+
+    widgets = (combo_box, use_preset_luma_combo, user_luma_select, user_luma_label, keys)
     
     combo_box.connect("changed", editable_property.combo_selection_changed, keys)
-    return _get_two_column_editor_row(editable_property.get_display_name(), combo_box)
+    use_preset_luma_combo.connect("changed", _wipe_preset_combo_changed, editable_property, widgets)
+    dialog.connect('response', _wipe_lumafile_dialog_response, editable_property, widgets)
+    
+    return editor_pane
+
+def _wipe_preset_combo_changed(widget, ep, widgets):
+    combo_box, use_preset_luma_combo, user_luma_select, user_luma_label, keys = widgets
+    if widget.get_active() == 1:
+        combo_box.set_sensitive(False)
+        user_luma_select.set_sensitive(True)
+        user_luma_label.set_sensitive(True)
+        file_name = user_luma_select.get_filename()
+        if file_name != None:
+            ep.write_value(file_name)
+    else:
+        user_luma_select.set_sensitive(False)
+        user_luma_label.set_sensitive(False)
+        combo_box.set_sensitive(True)
+        ep.combo_selection_changed(combo_box, keys)
+
+def _wipe_lumafile_dialog_response(dialog, response_id, ep, widgets):
+    combo_box, use_preset_luma_combo, user_luma_select, user_luma_label, keys = widgets
+    file_name = user_luma_select.get_filename()
+    if file_name != None:
+        ep.write_value(file_name)
 
 def _create_composite_editor(clip, editable_properties):
     aligned = filter(lambda ep: ep.name == "aligned", editable_properties)[0]
     distort = filter(lambda ep: ep.name == "distort", editable_properties)[0]
-    fill = filter(lambda ep: ep.name == "fill", editable_properties)[0]
     operator = filter(lambda ep: ep.name == "operator", editable_properties)[0]
     values = ["over","and","or","xor"]
     deinterlace = filter(lambda ep: ep.name == "deinterlace", editable_properties)[0]
     progressive = filter(lambda ep: ep.name == "progressive", editable_properties)[0]
     force_values = [_("Nothing"),_("Progressive"),_("Deinterlace"),_("Both")]
-    force_data = (deinterlace, progressive)
 
     combo_box = gtk.combo_box_new_text()
     for val in force_values:
@@ -332,20 +439,15 @@ def _create_composite_editor(clip, editable_properties):
     force_vbox.pack_start(gtk.Label(_("Force")), True, True, 0)
     force_vbox.pack_start(combo_box, True, True, 0)
 
-    pad1 = gtk.Label()
-    pad1.set_size_request(3,5)
-    pad2 = gtk.Label()
-    pad2.set_size_request(3,5)
-
     hbox = gtk.HBox(False, 4)
-    hbox.pack_start(pad1, False, False, 0)
+    hbox.pack_start(guiutils.get_pad_label(3, 5), False, False, 0)
     hbox.pack_start(_get_boolean_check_box_button_column(_("Align"), aligned), False, False, 0)
     hbox.pack_start(_get_boolean_check_box_button_column(_("Distort"), distort), False, False, 0)
     hbox.pack_start(gtk.Label(), True, True, 0)
     hbox.pack_start(_get_combo_box_column(_("Alpha"), values, operator), False, False, 0)
     hbox.pack_start(gtk.Label(), True, True, 0)
     hbox.pack_start(force_vbox, False, False, 0)
-    hbox.pack_start(pad2, False, False, 0)
+    hbox.pack_start(guiutils.get_pad_label(3, 5), False, False, 0)
     return hbox
 
 def _compositor_editor_force_combo_box_callback(combo_box, data):
@@ -365,6 +467,52 @@ def _compositor_editor_force_combo_box_callback(combo_box, data):
         deinterlace.write_value("1")
         progressive.write_value("1")
 
+def _create_rotion_geometry_editor(clip, editable_properties):
+    # Build a custom object that duck types for TransitionEditableProperty to use in editor
+    ep = utils.EmptyClass()
+    # pack real properties to go
+    ep.x = filter(lambda ep: ep.name == "x", editable_properties)[0]
+    ep.y = filter(lambda ep: ep.name == "y", editable_properties)[0]
+    ep.x_scale = filter(lambda ep: ep.name == "x scale", editable_properties)[0]
+    ep.y_scale = filter(lambda ep: ep.name == "y scale", editable_properties)[0]
+    ep.rotation = filter(lambda ep: ep.name == "rotation", editable_properties)[0]
+    ep.opacity = filter(lambda ep: ep.name == "opacity", editable_properties)[0]
+    # Screen width and height are needeed for frei0r conversions
+    ep.profile_width = current_sequence().profile.width()
+    ep.profile_height = current_sequence().profile.height()
+    # duck type methods, using opacity is not meaningful, any property with clip member could do
+    ep.get_clip_tline_pos = lambda : ep.opacity.clip.clip_in # clip is compositor, compositor in and out points staright in timeline frames
+    ep.get_clip_length = lambda : ep.opacity.clip.clip_out - ep.opacity.clip.clip_in + 1
+    ep.get_input_range_adjustment = lambda : gtk.Adjustment(float(100), float(0), float(100), float(1))
+    ep.get_display_name = lambda : "Opacity"
+    ep.get_pixel_aspect_ratio = lambda : (float(current_sequence().profile.sample_aspect_num()) / current_sequence().profile.sample_aspect_den())
+    ep.get_in_value = lambda out_value : out_value # hard coded for opacity 100 -> 100 range
+    ep.write_out_keyframes = lambda w_kf : keyframeeditor.rotating_ge_write_out_keyframes(ep, w_kf)
+    # duck type members
+    x_tokens = ep.x.value.split(";")
+    y_tokens = ep.y.value.split(";")
+    x_scale_tokens = ep.x_scale.value.split(";")
+    y_scale_tokens = ep.y_scale.value.split(";")
+    rotation_tokens = ep.rotation.value.split(";")
+    opacity_tokens = ep.opacity.value.split(";")
+    
+    value = ""
+    for i in range(0, len(x_tokens)): # these better match, same number of keyframes for all values, or this will not work
+        frame, x = x_tokens[i].split("=")
+        frame, y = y_tokens[i].split("=")
+        frame, x_scale = x_scale_tokens[i].split("=")
+        frame, y_scale = y_scale_tokens[i].split("=")
+        frame, rotation = rotation_tokens[i].split("=")
+        frame, opacity = opacity_tokens[i].split("=")
+        
+        frame_str = str(frame) + "=" + str(x) + ":" + str(y) + ":" + str(x_scale) + ":" + str(y_scale) + ":" + str(rotation) + ":" + str(opacity)
+        value += frame_str + ";"
+
+    ep.value = value.strip(";")
+
+    kf_edit = keyframeeditor.RotatingGeometryEditor(ep, False)
+    return kf_edit
+
 def _create_region_editor(clip, editable_properties):
     aligned = filter(lambda ep: ep.name == "composite.aligned", editable_properties)[0]
     distort = filter(lambda ep: ep.name == "composite.distort", editable_properties)[0]
@@ -373,7 +521,6 @@ def _create_region_editor(clip, editable_properties):
     deinterlace = filter(lambda ep: ep.name == "composite.deinterlace", editable_properties)[0]
     progressive = filter(lambda ep: ep.name == "composite.progressive", editable_properties)[0]
     force_values = [_("Nothing"),_("Progressive"),_("Deinterlace"),_("Both")]
-    force_data = (deinterlace, progressive)
 
     combo_box = gtk.combo_box_new_text()
     for val in force_values:
@@ -385,24 +532,47 @@ def _create_region_editor(clip, editable_properties):
     force_vbox.pack_start(gtk.Label(_("Force")), True, True, 0)
     force_vbox.pack_start(combo_box, True, True, 0)
 
-    pad1 = gtk.Label()
-    pad1.set_size_request(3,5)
-    pad2 = gtk.Label()
-    pad2.set_size_request(3,5)
-
     hbox = gtk.HBox(False, 4)
-    hbox.pack_start(pad1, False, False, 0)
+    hbox.pack_start(guiutils.get_pad_label(3, 5), False, False, 0)
     hbox.pack_start(_get_boolean_check_box_button_column(_("Align"), aligned), False, False, 0)
     hbox.pack_start(_get_boolean_check_box_button_column(_("Distort"), distort), False, False, 0)
     hbox.pack_start(gtk.Label(), True, True, 0)
     hbox.pack_start(_get_combo_box_column(_("Alpha"), values, operator), False, False, 0)
     hbox.pack_start(gtk.Label(), True, True, 0)
     hbox.pack_start(force_vbox, False, False, 0)
-    hbox.pack_start(pad2, False, False, 0)
+    hbox.pack_start(guiutils.get_pad_label(3, 5), False, False, 0)
     return hbox
+
+def _create_color_grader(filt, editable_properties):
+    color_grader = extraeditors.ColorGrader(editable_properties)
+
+    vbox = gtk.VBox(False, 4)
+    vbox.pack_start(gtk.Label(), True, True, 0)
+    vbox.pack_start(color_grader.widget, False, False, 0)
+    vbox.pack_start(gtk.Label(), True, True, 0)
+    vbox.no_separator = True
+    return vbox
+
+def _create_crcurves_editor(filt, editable_properties):
+    curves_editor = extraeditors.CatmullRomFilterEditor(editable_properties)
+
+    vbox = gtk.VBox(False, 4)
+    vbox.pack_start(curves_editor.widget, False, False, 0)
+    vbox.pack_start(gtk.Label(), True, True, 0)
+    vbox.no_separator = True
+    return vbox
+
+def _create_colorbox_editor(filt, editable_properties):
+    colorbox_editor = extraeditors.ColorBoxFilterEditor(editable_properties)
     
+    vbox = gtk.VBox(False, 4)
+    vbox.pack_start(colorbox_editor.widget, False, False, 0)
+    vbox.pack_start(gtk.Label(), True, True, 0)
+    vbox.no_separator = True
+    return vbox
+
 def _get_force_combo_index(deinterlace, progressive):
-    # these must correspond to hardcoded values ["Nothing","Progressive","Deinterlace","Both"] above
+    # These correspond to hardcoded values ["Nothing","Progressive","Deinterlace","Both"] above
     if int(deinterlace.value) == 0:
         if int(progressive.value) == 0:
             return 0
@@ -509,5 +679,23 @@ EDITOR_ROW_CREATORS = { \
     CLIP_FRAME_SLIDER: lambda ep: _get_clip_frame_slider(ep),
     NO_EDITOR: lambda ep: _get_no_editor(),
     COMPOSITE_EDITOR_BUILDER: lambda comp, editable_properties: _create_composite_editor(comp, editable_properties),
-    REGION_EDITOR_BUILDER: lambda comp, editable_properties: _create_region_editor(comp, editable_properties)
+    REGION_EDITOR_BUILDER: lambda comp, editable_properties: _create_region_editor(comp, editable_properties),
+    ROTATION_GEOMETRY_EDITOR_BUILDER: lambda comp, editable_properties: _create_rotion_geometry_editor(comp, editable_properties),
+    COLOR_CORRECTOR: lambda filt, editable_properties: _create_color_grader(filt, editable_properties),
+    CR_CURVES: lambda filt, editable_properties:_create_crcurves_editor(filt, editable_properties),
+    COLOR_BOX: lambda filt, editable_properties:_create_colorbox_editor(filt, editable_properties)
     }
+
+"""
+    # example code for using slider editor with NON-MLT property
+    #hue = filter(lambda ep: ep.name == "hue", editable_properties)[0]
+    #hue_row = _get_slider_row(hue, None, True)
+    #saturation = filter(lambda ep: ep.name == "saturation", editable_properties)[0]
+    #saturation_row = _get_slider_row(saturation, None, True)
+    #value = filter(lambda ep: ep.name == "value", editable_properties)[0]
+    #value_row = _get_slider_row(value, None, True)
+    #colorbox_editor = extraeditors.ColorBoxFilterEditor(editable_properties, [hue_row, saturation_row, value_row])
+    #hue.adjustment_listener = colorbox_editor.hue_changed
+    #saturation.adjustment_listener = colorbox_editor.saturation_changed
+    #value.adjustment_listener = colorbox_editor.value_changed
+"""
