@@ -23,22 +23,16 @@ Module contains class Sequence that is the multitrack media object being edited
 by the application. A project has 1-n of these.
 """
 
-import copy
-import gnomevfs
 import mlt
-import time #added when testing 
-import types
 import os
 
 import appconsts
 import edit
 import editorstate
-import editorpersistance
-import memoryleak
 import mltfilters
 import mlttransitions
-import respaths
-import resync
+import mltrefhold
+import patternproducer
 import utils
 
 # Media types for tracks or clips
@@ -46,6 +40,7 @@ UNKNOWN = appconsts.UNKNOWN
 VIDEO = appconsts.VIDEO
 AUDIO = appconsts.AUDIO
 IMAGE = appconsts.IMAGE
+IMAGE_SEQUENCE = appconsts.IMAGE_SEQUENCE
 RENDERED_VIDEO = appconsts.RENDERED_VIDEO
 PATTERN_PRODUCER = appconsts.PATTERN_PRODUCER
 FILE_DOES_NOT_EXIST = appconsts.FILE_DOES_NOT_EXIST
@@ -61,6 +56,9 @@ LOCKED = appconsts.LOCKED # no edits allowed
 TRACK_HEIGHT_NORMAL = appconsts.TRACK_HEIGHT_NORMAL # track height in canvas and column
 TRACK_HEIGHT_SMALL = appconsts.TRACK_HEIGHT_SMALL # track height in canvas and column
 
+# pan magic value indicating that no pan is applied
+NO_PAN = appconsts.NO_PAN #-99
+
 # MLT types
 MLT_PLAYLIST = 0
 MLT_PRODUCER = 1
@@ -69,7 +67,7 @@ MLT_FILTER = 2
 # Number of tracks available
 # NOTE: These are set from other modules (and this one when cloning) when creating or loading projects
 # and used in Sequence.__init__(...) when creating sequences.
-# Hacky design, tracks count should be provided via constructor at creation time.
+# Weak design, tracks count should be provided via constructor at creation time.
 AUDIO_TRACKS_COUNT = 4
 VIDEO_TRACKS_COUNT = 5
 
@@ -92,6 +90,8 @@ black_track_clip = None
 # USED FOR TRACKS, NOT CLIPS. Clips handled using values in appconsts.py
 MUTE_STATES = [(True, True), (False, True), (True, False), (False, False)]
 
+# Track that all audio is mixed down to combine for output.
+AUDIO_MIX_DOWN_TRACK = 0
 
 
 class Sequence:
@@ -104,11 +104,16 @@ class Sequence:
         self.name = name # name of sequence
         self.next_id = 0 # id for next created clip
         self.profile = profile
+        self.master_audio_gain = 1.0
+        self.master_audio_pan = NO_PAN
         self.tracks = []
         self.compositors = []
-        self.markers = [] #future feature, not used currently
+        self.markers = [] # markers are tuples (name_str, frame_int)
         self.proxyclips = {} #future feature, not used currently
         self.rendered_versions = {} #future feature, not used currently
+        self.watermark_filter = None
+        self.watermark_file_path = None
+        self.seq_len = 0 # used in trim crash hack, remove when fixed
 
         # MLT objects for a multitrack sequence
         self.init_mlt_objects()
@@ -121,13 +126,35 @@ class Sequence:
         self.tractor.mark_in = -1
         self.tractor.mark_out = -1
 
+        # Only create and add pan filter if actual pan is applied
+        # This method gets called on load and we only want to add filter then if pan is applied,
+        # and not on initial creation.
+        # audiomonitoring.py calls add_track_pan_filter() when pan turned on for initial creation
+        if self.master_audio_pan != NO_PAN:
+            self.add_track_pan_filter(self.tractor, self.master_audio_pan)
+
+        # Create and ad gain filter
+        gain_filter = mlt.Filter(self.profile, "volume")
+        mltrefhold.hold_ref(gain_filter)
+        gain_filter.set("gain", str(self.master_audio_gain))
+        self.tractor.attach(gain_filter)
+        self.tractor.gain_filter = gain_filter
+        
         self.field = self.tractor.field()
         self.multitrack = self.tractor.multitrack()
         
         self.vectorscope = mlt.Filter(self.profile, "frei0r.vectorscope")
+        mltrefhold.hold_ref(self.vectorscope)
+        self.vectorscope.set("mix", "0.5")
+        self.vectorscope.set("overlay sides", "0.0") 
         self.rgbparade =  mlt.Filter(self.profile, "frei0r.rgbparade")
+        mltrefhold.hold_ref(self.rgbparade)
+        self.rgbparade.set("mix", "0.4")
+        self.rgbparade.set("overlay sides", "0.0") 
         self.outputfilter = None
-    
+  
+
+
     # ---------------------------------------- tracks
     def create_default_tracks(self):
         """
@@ -150,14 +177,13 @@ class Sequence:
         self.add_track(VIDEO)
         
         # Audio tracks
-        #audio_tracks = AUDIO_TRACKS_COUNT
         for i in range(0, AUDIO_TRACKS_COUNT):
             track = self.add_track(AUDIO)
             track.height = TRACK_HEIGHT_SMALL
         
         # Video tracks
         self.first_video_index = AUDIO_TRACKS_COUNT + 1 # index of first editable video track
-        #video_tracks = VIDEO_TRACKS_COUNT
+
         for i in range(0, VIDEO_TRACKS_COUNT):
             self.add_track(VIDEO) # editable
             if i > 0:
@@ -174,30 +200,31 @@ class Sequence:
         # below can be viewed.
         self.add_track(VIDEO, True) 
 
+        self._create_black_track_clip()
+        
+        # Add black clip to black bg track
+        self.tracks[0].clips.append(black_track_clip) # py
+        self.tracks[0].append(black_track_clip, 0, 0) # mlt
+
+    def _create_black_track_clip(self):
         # Create 1 fr long black bg clip and set in and out
         global black_track_clip # btw, why global?
         pattern_producer_data = utils.EmptyClass()
-        pattern_producer_data.patter_producer_type = appconsts.COLOR_CLIP
+        pattern_producer_data.patter_producer_type = patternproducer.COLOR_CLIP
         pattern_producer_data.gdk_color_str = "#000000000000"
         pattern_producer_data.name = "black_bg"
         black_track_clip = self.create_pattern_producer(pattern_producer_data)
-        clip_in = 0
-        clip_out = 0
-        black_track_clip.clip_in = clip_in
-        black_track_clip.clip_out = clip_out
+        black_track_clip.clip_in = 0
+        black_track_clip.clip_out = 0
 
-        # Add black clip to black bg track
-        self.tracks[0].clips.append(black_track_clip) # py
-        self.tracks[0].append(black_track_clip, clip_in, clip_out) # mlt
-
-    def add_track(self, type, is_hidden=False):
+    def add_track(self, track_type, is_hidden=False):
         """ 
         Creates a MLT playlist object, adds project
         data and adds to tracks list.
         """
         new_track = mlt.Playlist()
 
-        self._add_track_attributes(new_track, type)
+        self._add_track_attributes(new_track, track_type)
         new_track.is_sync_track = False
 
         # Connect to MLT multitrack
@@ -209,7 +236,7 @@ class Sequence:
         
         # Mix all audio to track 1 by combining them one after another 
         # using an always active field transition.
-        if ((new_track.id > 1) # black bg or track1 it's self does not need to be mixed
+        if ((new_track.id > AUDIO_MIX_DOWN_TRACK) # black bg or track1 it's self does not need to be mixed
             and (is_hidden == False)): # We actually do want hidden track to cover all audio below, which happens if it is not mixed.
             self._mix_audio_for_track(new_track)
         
@@ -218,16 +245,6 @@ class Sequence:
         
         return new_track
 
-    def _mix_audio_for_track(self, track):
-        transition = mlt.Transition(self.profile, "mix")
-        transition.set("in", 0)
-        transition.set("out", 0)
-        transition.set("a_track", 1)
-        transition.set("b_track", track.id)
-        transition.set("always_active", 1)
-        transition.set("combine", 1)
-        self.field.plant_transition(transition, 1, track.id)
-
     def _add_track_attributes(self, track, type):                
         # Add data attr
         track.type = type
@@ -251,20 +268,103 @@ class Sequence:
         if editorstate.SCREEN_HEIGHT < 863:# Fix for 786 screens
             track.height = TRACK_HEIGHT_SMALL
         
+        # Audio gain and pan values, these are overwritten later with saved values when loading 
+        track.audio_gain = 1.0 # active range 0 - 1
+        track.audio_pan = NO_PAN # active range 0-1, 0.5 is middle
+        
         # Tracks may be FREE or LOCKED
         track.edit_freedom = FREE
-  
+
+    def _mix_audio_for_track(self, track):
+        # Create and add transition to combine track audios
+        transition = mlt.Transition(self.profile, "mix")
+        mltrefhold.hold_ref(transition)
+        transition.set("a_track", int(AUDIO_MIX_DOWN_TRACK))
+        transition.set("b_track", track.id)
+        transition.set("always_active", 1)
+        transition.set("combine", 1)
+        self.field.plant_transition(transition, int(AUDIO_MIX_DOWN_TRACK), track.id)
+
+        # Create and ad gain filter
+        gain_filter = mlt.Filter(self.profile, "volume")
+        mltrefhold.hold_ref(gain_filter)
+        gain_filter.set("gain", str(track.audio_gain))
+        track.attach(gain_filter)
+        track.gain_filter = gain_filter
+
+        # Add pan filter if this track is panorated
+        if track.audio_pan != NO_PAN:
+            self.add_track_pan_filter(track, 0.5) 
+            track.audio_pan = 0.5
+
+    def minimize_tracks_height(self):
+        for i in range (1, len(self.tracks) - 1):# visible tracks
+            track = self.tracks[i]
+            track.height = TRACK_HEIGHT_SMALL
+
+    def maximize_tracks_height(self, allocation):
+        for i in range (1, len(self.tracks) - 1):# visible tracks
+            track = self.tracks[i]
+            track.height = TRACK_HEIGHT_NORMAL
+    
+        self.resize_tracks_to_fit(allocation)
+
+    def maximize_video_tracks_height(self, allocation):
+        self.minimize_tracks_height()
+        for i in range (self.first_video_index, len(self.tracks) - 1):# visible tracks
+            track = self.tracks[i]
+            track.height = TRACK_HEIGHT_NORMAL
+    
+        self.resize_tracks_to_fit(allocation)
+
+    def maximize_audio_tracks_height(self, allocation):
+        self.minimize_tracks_height()
+        for i in range (1, self.first_video_index):
+            track = self.tracks[i]
+            track.height = TRACK_HEIGHT_NORMAL
+    
+        self.resize_tracks_to_fit(allocation)
+
     def get_tracks_height(self):
         h = 0
         for i in range (1, len(self.tracks) - 1):# visible tracks
             track = self.tracks[i]
             h += track.height
         return  h
-        # Add method that returns audio sync track index name
+
+    def set_track_gain(self, track, gain):
+        track.gain_filter.set("gain", str(gain))
+        track.audio_gain = gain
+
+    def set_master_gain(self, gain):
+        self.tractor.gain_filter.set("gain", str(gain))
+        self.master_audio_gain = gain
+
+    def add_track_pan_filter(self, track, value):
+        # This method is used for master too, and called with tractor then
+        pan_filter = mlt.Filter(self.profile, "panner")
+        mltrefhold.hold_ref(pan_filter)
+        pan_filter.set("start", value)
+        track.attach(pan_filter)
+        track.pan_filter = pan_filter 
+
+    def set_track_pan_value(self, track, value):
+        track.pan_filter.set("start", str(value))
+        track.audio_pan = value
+    
+    def remove_track_pan_filter(self, track):
+        # This method is used for master too, and called with tractor then
+        track.detach(track.pan_filter)
+        track.pan_filter = None
+        track.audio_pan = NO_PAN
+
+    def set_master_pan_value(self, value):
+        self.tractor.pan_filter.set("start", str(value))
+        self.master_audio_pan = value
 
     def first_video_track(self):
         return self.tracks[self.first_video_index]
-    
+
     def all_tracks_off(self):
         for i in range (1, len(self.tracks) - 1):
             track = self.tracks[i]
@@ -278,7 +378,8 @@ class Sequence:
         Creates MLT Producer and adds attributes to it, but does 
         not add it to track/playlist object.
         """
-        producer = mlt.Producer(self.profile, path) # this runs 0.5s+ on some clips
+        producer = mlt.Producer(self.profile, str(path)) # this runs 0.5s+ on some clips
+        mltrefhold.hold_ref(producer)
         producer.path = path
         producer.filters = []
         
@@ -289,6 +390,7 @@ class Sequence:
             producer.name = new_clip_name
         producer.media_type = get_media_type(path)
         if producer.media_type == FILE_DOES_NOT_EXIST:
+            print "file does not exist"
             return None
 
         self.add_clip_attr(producer)
@@ -302,7 +404,8 @@ class Sequence:
         """
         fr_path = "framebuffer:" + path + "?" + str(speed)
         producer = mlt.Producer(self.profile, None, str(fr_path)) # this runs 0.5s+ on some clips
- 
+        mltrefhold.hold_ref(producer)
+
         (folder, file_name) = os.path.split(path)
         (name, ext) = os.path.splitext(file_name)
         producer.name = name
@@ -320,29 +423,15 @@ class Sequence:
         """
         pattern_producer_data is instance of projectdata.BinColorClip
         """
-        if pattern_producer_data.patter_producer_type == appconsts.COLOR_CLIP:
-            clip = self._create_color_clip(pattern_producer_data.gdk_color_str,
-                                          pattern_producer_data.name)
-        
-        # Save creation data (instance of projectdata.BinColorClip) for cloning when editing or doing save/load 
-        clip.create_data = copy.copy(pattern_producer_data)
-        clip.create_data.icon = None # this is not pickleable, recreate when needed
+        clip = patternproducer.create_pattern_producer(self.profile, pattern_producer_data)
+        self.add_clip_attr(clip)
         return clip
 
-    def _create_color_clip(self, gdk_color_str, name):
-        mlt_color = utils.gdk_color_str_to_mlt_color_str(gdk_color_str)
-
-        producer = mlt.Producer(self.profile, "colour", mlt_color)
-        producer.path = ""
-        producer.filters = []
-        producer.gdk_color_str = gdk_color_str
-
-        producer.name = name
-        producer.media_type = PATTERN_PRODUCER
-        self.add_clip_attr(producer)
-
-        return producer
-
+    def create_rendered_transition_clip(self, path, rendered_type):
+        clip = self.create_file_producer_clip(path)
+        clip.rendered_type = rendered_type
+        return clip
+    
     def add_clip_attr(self, clip):
         """
         File producers, transitions and black clips have same
@@ -357,10 +446,23 @@ class Sequence:
         clip.sync_data = None 
         clip.mute_filter = None #
         clip.stream_indexes = None # a, v stream indexes when not muted
-        clip.clip_length = lambda: _clip_length(clip)
+        clip.clip_length = lambda: _clip_length(clip) # MLT get_length gives wrong values for blanks
         clip.waveform_data = None
-    
-    def clone_clip_range_and_filters(self, clip, clone_clip):
+        clip.color = None # None means that clip type default color is displayed
+
+    def clone_track_clip(self, track, index):
+        orig_clip = track.clips[index]
+        return self.create_clone_clip(orig_clip)
+
+    def create_clone_clip(self, clip):
+        if clip.media_type != appconsts.PATTERN_PRODUCER:
+            clone_clip = self.create_file_producer_clip(clip.path) # file producer
+        else:
+            clone_clip = self.create_pattern_producer(clip.create_data) # pattern producer
+        self.clone_clip_and_filters(clip, clone_clip)
+        return clone_clip
+        
+    def clone_clip_and_filters(self, clip, clone_clip):
         """
         Clones clip range properties and filters that are needed for clip to be
         used in another clip's place, but not id, master_clip and selection
@@ -388,7 +490,7 @@ class Sequence:
         """
         self.next_id += 1
         return self.next_id - 1
-        
+
     # ------------------------------------------ blanks
     def create_and_insert_blank(self, track, index, length):
         """
@@ -408,7 +510,7 @@ class Sequence:
         """
         Used in hack for trim editing last clip of a track.
         """
-        clip = edit._remove_clip(track, len(track.clips) - 1)
+        edit._remove_clip(track, len(track.clips) - 1)
 
     # ------------------------------------------ filters
     def create_filter(self, filter_info):
@@ -422,8 +524,8 @@ class Sequence:
         return filter_object
 
     # ------------------------------------------------------ compositors
-    def create_compositor(self, compositor_type_index):
-        compositor = mlttransitions.create_compositor(compositor_type_index)
+    def create_compositor(self, compositor_type):
+        compositor = mlttransitions.create_compositor(compositor_type)
         compositor.create_mlt_objects(self.profile)
         return compositor
 
@@ -448,11 +550,11 @@ class Sequence:
 
     def _create_and_plant_clone_compositor(self, old_compositor):
         # Remove old compositor
-        edit.old_compositors.append(old_compositor) # HACK. Garbage collecting compositors causes crashes.
+        #edit.old_compositors.append(old_compositor) # HACK. Garbage collecting compositors causes crashes.
         self.field.disconnect_service(old_compositor.transition.mlt_transition)
         
         # Create and plant new compositor
-        compositor = self.create_compositor(old_compositor.compositor_index)
+        compositor = self.create_compositor(old_compositor.type_id)
         compositor.clone_properties(old_compositor)
         compositor.set_in_and_out(old_compositor.clip_in, old_compositor.clip_out)
         compositor.transition.set_tracks(old_compositor.transition.a_track, old_compositor.transition.b_track)
@@ -464,7 +566,6 @@ class Sequence:
         new_compositors = []
         video_diff = self.first_video_index - from_sequence.first_video_index
         for old_compositor in from_sequence.compositors:
-            print old_compositor.transition.b_track 
             if old_compositor.transition.b_track + video_diff < len(self.tracks) - 1:
                 clone_compositor = self._create_and_plant_clone_compositor_for_sequnce_clone(old_compositor, track_delta)
                 new_compositors.append(clone_compositor)
@@ -473,10 +574,10 @@ class Sequence:
     def _create_and_plant_clone_compositor_for_sequnce_clone(self, old_compositor, track_delta):
         # Used when cloning compositors to change track count by cloning sequence
         # Remove old compositor
-        edit.old_compositors.append(old_compositor) # HACK. Garbage collecting compositors causes crashes.
+        # edit.old_compositors.append(old_compositor) # HACK. Garbage collecting compositors causes crashes.
         
         # Create and plant new compositor
-        compositor = self.create_compositor(old_compositor.compositor_index)
+        compositor = self.create_compositor(old_compositor.type_id)
         compositor.clone_properties(old_compositor)
         compositor.set_in_and_out(old_compositor.clip_in, old_compositor.clip_out)
         compositor.transition.set_tracks(old_compositor.transition.a_track + track_delta, old_compositor.transition.b_track + track_delta)
@@ -490,7 +591,7 @@ class Sequence:
         self.compositors.append(compositor)
 
     def remove_compositor(self, old_compositor):
-        edit.old_compositors.append(old_compositor)# HACK. Garbage collecting compositors causes crashes.
+        #edit.old_compositors.append(old_compositor)# HACK. Garbage collecting compositors causes crashes.
         try:
             self.compositors.remove(old_compositor)
         except ValueError: # has been restacked since creation, needs to looked up using destroy_id
@@ -499,7 +600,7 @@ class Sequence:
                 if comp.destroy_id == old_compositor.destroy_id:
                     found = True
                     self.compositors.remove(comp)
-                    edit.old_compositors.append(comp)
+                    #edit.old_compositors.append(comp)
                     old_compositor = comp
             if found == False:
                 raise ValueError('compositor not found using destroy_id')
@@ -523,12 +624,16 @@ class Sequence:
         """
         Adds media clip to hidden track for viewing and for setting mark
         in and mark out points.
+        pattern_producer_data is MediaFile or AbstractPatternProduer object
         """
         track = self.tracks[-1] # Always last track
         if pattern_producer_data == None:
             self.monitor_clip = self.create_file_producer_clip(path)
         else:
-            self.monitor_clip = self.create_pattern_producer(pattern_producer_data)
+            if pattern_producer_data.type == IMAGE_SEQUENCE:
+                self.monitor_clip = self.create_file_producer_clip(pattern_producer_data.path)
+            else:
+                self.monitor_clip = self.create_pattern_producer(pattern_producer_data)
         
         edit._insert_clip(track, self.monitor_clip, 0, 0, \
                           self.monitor_clip.get_length() - 1)
@@ -539,6 +644,7 @@ class Sequence:
         """
         Adds clip to hidden track for trim editing display.
         """
+        self.tracks[-1].clear() # # TRIM INIT CRASH HACK, see clear_hidden_track there may be blank clip here
         track = self.tracks[-1] # Always last track
         
         # Display trimmmed clip on hidden track by creating copy of it.
@@ -551,7 +657,7 @@ class Sequence:
                 edit._insert_clip(track, clip, 1, 0, clip.get_length() - 1)
             else:
                 edit._insert_clip(track, clip, 1, -clip_start_pos, clip.get_length() - 1) # insert index 1 ?
-        # Pattern producer
+        # Pattern producer (FIX ME: does not allow for keyframes in pattern producer)
         else:
             clip = self.create_pattern_producer(patter_producer_data)
             edit._insert_clip(track, clip, 0, 0, clip.get_length() - 1)
@@ -586,10 +692,49 @@ class Sequence:
         Last track is hidden track used to display clips and trim edits.
         Here that track is cleared of any content.
         """
-        self.tracks[-1].clear()
+        self.update_edit_tracks_length()
+
+        # Empty timeline needs blank clip of len atleast 1 because  
+        # edit_insert_blank() always needs a clip to add attributes to 
+        # and that method is fundamendal and cannot be changed. 
+        seq_len = self.seq_len
+        if seq_len < 1:
+            seq_len = 1
+        
         self.tracks[-1].clips = []
+        self.tracks[-1].clear()
+
+        edit._insert_blank(self.tracks[-1], 0, seq_len) # TRIM INIT CRASH HACK. This being empty crashes a lot, so far unexplained.
+        
         self._unmute_editable()
-    
+
+    def update_edit_tracks_length(self):
+        # NEEDED FOR TRIM CRASH HACK, REMOVE IF FIXED
+        self.seq_len = 0
+        for i in range(1, len(self.tracks) - 1):
+            track_len = self.tracks[i].get_length()
+            if track_len > self.seq_len:
+                self.seq_len = track_len
+
+    def update_trim_hack_blank_length(self):
+        # NEEDED FOR TRIM CRASH HACK, REMOVE IF FIXED
+        self.tracks[-1].clips = []
+        self.tracks[-1].clear()
+
+        seq_len = self.seq_len
+        if seq_len < 1:
+            seq_len = 1
+            
+        edit._insert_blank(self.tracks[-1], 0, seq_len)
+
+    def get_seq_range_frame(self, frame):
+        # NEEDED FOR TRIM CRASH HACK, REMOVE IF FIXED
+        # remove TimeLineFrameScale then too
+        if frame >= (self.seq_len - 1):
+            return self.seq_len - 1
+        else:
+            return frame
+
     def _mute_editable(self):
         for i in range(1, len(self.tracks) - 1):
             track = self.tracks[i]
@@ -615,18 +760,36 @@ class Sequence:
             self.tractor.attach(self.rgbparade)
             self.outputfilter = self.rgbparade
 
+    # ---------------------------------------------------- def add watermark
+    def add_watermark(self, watermark_file_path):
+        watermark = mlt.Filter(self.profile, "watermark")
+        mltrefhold.hold_ref(watermark)
+        watermark.set("resource",str(watermark_file_path))
+        watermark.set("composite.always_active", 1)
+        self.tractor.attach(watermark)
+        self.watermark_filter = watermark
+        self.watermark_file_path = watermark_file_path
+
+    def remove_watermark(self):
+        self.tractor.detach(self.watermark_filter)
+        self.watermark_filter = None
+        self.watermark_file_path = None
+
     # ------------------------------------------------ length, seek, misc
     def update_length(self):
         """
         Set black to track length of sequence.
         """
-        global black_track_clip 
+        global black_track_clip
+        if black_track_clip == None: # This fails for launch with assoc Gnome file because this has not been made yet.
+                                     # This global black_track_clip is brain dead.  
+            self._create_black_track_clip()
         c_in = 0
         c_out = self.get_length()
         black_track_clip.clip_in = c_in
         black_track_clip.clip_out = c_out
         black_track_clip.set_in_and_out(c_in, c_out)
- 
+
     def get_length(self):
         return self.multitrack.get_length()
 
@@ -641,7 +804,7 @@ class Sequence:
             elif count + 1 == self.first_video_index:
                 # This shold not happen because track heights should be set up so that minimized app 
                 # has enough space to display all tracks.
-                # Yet it happens sometimes
+                # Yet it happens sometimes, meh.
                 print "sequence.resize_tracks_to_fit (): could not make panels fit"
                 fix_next = False
             else:
@@ -697,7 +860,7 @@ class Sequence:
             if clip_start_frame == tline_frame:
                 index = index - 1
             
-            # Check index
+            # Check index is good
             try:
                 clip = track.clips[index]            
             except Exception:
@@ -739,8 +902,8 @@ class Sequence:
         This is done in a way that the user sees the track displayed as top most
         on screen being the first active when doing for e.g. a monitor insert.
         track: 0, black bg video
-        tracks: 1 - (self.first_video_index - 1), audio, numbred to user in oppposite direction as 1 - n (user_index = self.first_video_index - index)
-        tracks: self.first_video_index - (len - 2), video, numbred to user as 1 - n (user_index = index - self.first_video_index + 1)
+        tracks: 1 - (self.first_video_index - 1), audio, numbered to user in opposite direction as 1 - n (user_index = self.first_video_index - index)
+        tracks: self.first_video_index - (len - 2), video, numbered to user as 1 - n (user_index = index - self.first_video_index + 1)
         track: (len - 1). hidden video track for trim and clip display
         """
         # Video
@@ -805,7 +968,6 @@ class Sequence:
             print i, ": id:", clip.id, " in:",clip.clip_in," out:", \
             clip.clip_out, msg
 
-
         print "MLT"
         for i in range(0, track.count()):
             clip = track.get_clip(i)
@@ -823,14 +985,22 @@ class Sequence:
 def get_media_type(file_path):
     """
     Returns media type of file.
-    """ 
-    try:
-        mime_type = gnomevfs.get_mime_type(file_path)
-    except Exception, err:
-        if not os.path.exists(file_path):
-            return FILE_DOES_NOT_EXIST
-        else:
-            return UNKNOWN
+    """
+    if os.path.exists(file_path):
+        mime_type = utils.get_file_type(file_path)
+    else:
+        # IMAGE_SEQUENCE media objects have a MLT formatted resource path that does not
+        # point to an existing file in the file system. 
+        # We're doing a heuristic here to identify those.
+        pros_index = file_path.find("%0")
+        d_index = file_path.find("d.")
+        if pros_index != -1 and d_index != -1:
+            return IMAGE_SEQUENCE
+        all_index = file_path.find(".all")
+        if all_index != -1:
+            return IMAGE_SEQUENCE
+            
+        return FILE_DOES_NOT_EXIST
         
     if mime_type.startswith("video"):
         return VIDEO
@@ -842,7 +1012,7 @@ def get_media_type(file_path):
         return IMAGE
     
     return UNKNOWN
-    
+
 def _clip_length(clip):
     return clip.clip_out - clip.clip_in + 1
 
@@ -895,8 +1065,14 @@ def _clone_for_fewer_tracks(old_seq, new_seq):
 def _clone_tracks(old_seq, new_seq, first_to_track_index, last_to_track_index, first_from_track_index):
     from_track_index = first_from_track_index
     for i in range(first_to_track_index, last_to_track_index + 1):
+        if from_track_index > len(old_seq.tracks) - 1: # when changing to a (8V,1A) tracks sequence this range needs to be checked for
+            continue 
         from_track = old_seq.tracks[from_track_index]
+        
+        if i > len(new_seq.tracks) - 1: # when changing to a (1V,8A) tracks sequence this range needs to be checked for
+            continue
         to_track = new_seq.tracks[i]
+
         _copy_track_contents(from_track, to_track, new_seq)
         from_track_index = from_track_index + 1
     
