@@ -24,10 +24,9 @@ import os
 import pango
 import pangocairo
 import pickle
-import time
 import threading
 
-import dialogs
+import toolsdialogs
 from editorstate import PLAYER
 from editorstate import PROJECT
 import editorstate
@@ -79,8 +78,16 @@ def show_titler():
 
 def close_titler():
     global _titler, _titler_data
-    _titler.window.destroy()
+    
+    _titler.set_visible(False)
+    while(gtk.events_pending()):
+        gtk.main_iteration()
+        
+    _titler.destroy()
+    while(gtk.events_pending()):
+        gtk.main_iteration()
     _titler = None
+
     if not _keep_titler_data:
         _titler_data = None
 
@@ -107,6 +114,7 @@ class TextLayer:
         self.drop_shadow = None # future feature, here to keep file compat once added
         self.animation = None # future feature
         self.layer_attributes = None # future feature (kerning etc. go here, we're not using all pango features)
+        self.visible = True
 
     def get_font_desc_str(self):
         return self.font_family + " " + self.font_face + " " + str(self.font_size)
@@ -162,6 +170,8 @@ class Titler(gtk.Window):
         self.view_editor = vieweditor.ViewEditor(PLAYER().profile, VIEW_EDITOR_WIDTH, VIEW_EDITOR_HEIGHT)
         self.view_editor.active_layer_changed_listener = self.active_layer_changed
         
+        self.guides_toggle = vieweditor.GuidesViewToggle(self.view_editor)
+        
         add_b = gtk.Button(_("Add"))
         del_b = gtk.Button(_("Delete"))
         add_b.connect("clicked", lambda w:self._add_layer_pressed())
@@ -179,8 +189,8 @@ class Titler(gtk.Window):
         center_v = gtk.Button()
         center_v.set_image(center_v_icon)
         center_v.connect("clicked", lambda w:self._center_v_pressed())
-            
-        self.layer_list = TextLayerListView(self._layer_selection_changed)
+
+        self.layer_list = TextLayerListView(self._layer_selection_changed, self._layer_visibility_toggled)
         self.layer_list.set_size_request(TEXT_LAYER_LIST_WIDTH, TEXT_LAYER_LIST_HEIGHT)
     
         self.text_view = gtk.TextView()
@@ -318,19 +328,22 @@ class Titler(gtk.Window):
         next_frame.set_image(next_icon)
         next_frame.connect("clicked", lambda w:self._next_frame_pressed())
 
+        self.scale_selector = vieweditor.ScaleSelector(self)
+
         timeline_box = gtk.HBox()
         timeline_box.pack_start(guiutils.get_in_centering_alignment(self.tc_display.widget), False, False, 0)
         timeline_box.pack_start(guiutils.get_in_centering_alignment(pos_bar_frame, 1.0), True, True, 0)
         timeline_box.pack_start(prev_frame, False, False, 0)
         timeline_box.pack_start(next_frame, False, False, 0)
-        timeline_box.pack_start(vieweditor.ScaleSelector(self), False, False, 0)
+        timeline_box.pack_start(self.guides_toggle, False, False, 0)
+        timeline_box.pack_start(self.scale_selector, False, False, 0)
         
         positions_box = gtk.HBox()
         positions_box.pack_start(gtk.Label(), True, True, 0)
-        positions_box.pack_start(gtk.Label("X pos"), False, False, 0)
+        positions_box.pack_start(gtk.Label("X:"), False, False, 0)
         positions_box.pack_start(self.x_pos_spin, False, False, 0)
         positions_box.pack_start(guiutils.pad_label(10, 5), False, False, 0)
-        positions_box.pack_start(gtk.Label("Y pos"), False, False, 0)
+        positions_box.pack_start(gtk.Label("Y:"), False, False, 0)
         positions_box.pack_start(self.y_pos_spin, False, False, 0)
         positions_box.pack_start(guiutils.pad_label(10, 5), False, False, 0)
         #positions_box.pack_start(gtk.Label(_("Angle")), False, False, 0)
@@ -393,7 +406,7 @@ class Titler(gtk.Window):
 
         editor_row = gtk.HBox()
         editor_row.pack_start(controls_panel, False, False, 0)
-        editor_row.pack_start(editor_panel, False, False, 0)
+        editor_row.pack_start(editor_panel, True, True, 0)
 
         alignment = gtk.Alignment(0.5, 0.5, 1.0, 1.0)
         alignment.set_padding(8,8,8,8)
@@ -405,6 +418,9 @@ class Titler(gtk.Window):
         self._update_gui_with_active_layer_data()
         self.show_all()
 
+        self.connect("size-allocate", lambda w, e:self.window_resized())
+        self.connect("window-state-event", lambda w, e:self.window_resized())
+    
     def load_titler_data(self):
         # clear and then load layers, and set layer 0 active
         self.view_editor.clear_layers()
@@ -432,7 +448,11 @@ class Titler(gtk.Window):
         self.tc_display.set_frame(frame)
         self.pos_bar.widget.queue_draw()
         self._update_active_layout()
-        
+
+    def window_resized(self):
+        scale = self.scale_selector.get_current_scale()
+        self.scale_changed(scale)
+
     def scale_changed(self, new_scale):
         self.view_editor.set_scale_and_update(new_scale)
         self.view_editor.edit_area.queue_draw()
@@ -452,7 +472,7 @@ class Titler(gtk.Window):
         self.show_current_frame()
 
     def _save_title_pressed(self):
-        dialogs.save_titler_graphic_as_dialog(self._save_title_dialog_callback, "title.png", None)
+        toolsdialogs.save_titler_graphic_as_dialog(self._save_title_dialog_callback, "title.png", None)
 
     def _save_title_dialog_callback(self, dialog, response_id):
         if response_id == gtk.RESPONSE_ACCEPT:
@@ -478,7 +498,7 @@ class Titler(gtk.Window):
             dialog.destroy()
 
     def _save_layers_pressed(self):
-        dialogs.save_titler_data_as_dialog(self._save_layers_dialog_callback, "titler_layers", None)
+        toolsdialogs.save_titler_data_as_dialog(self._save_layers_dialog_callback, "titler_layers", None)
 
     def _save_layers_dialog_callback(self, dialog, response_id):
         if response_id == gtk.RESPONSE_ACCEPT:
@@ -490,7 +510,7 @@ class Titler(gtk.Window):
             dialog.destroy()
             
     def _load_layers_pressed(self):
-        dialogs.load_titler_data_dialog(self._load_layers_dialog_callback)
+        toolsdialogs.load_titler_data_dialog(self._load_layers_dialog_callback)
         
     def _load_layers_dialog_callback(self, dialog, response_id):
         if response_id == gtk.RESPONSE_ACCEPT:
@@ -574,6 +594,14 @@ class Titler(gtk.Window):
         self.layer_list.fill_data_model()
         self._activate_layer(0)
 
+    def _layer_visibility_toggled(self, layer_index):
+        toggled_visible = (self.view_editor.edit_layers[layer_index].visible == False)
+        self.view_editor.edit_layers[layer_index].visible = toggled_visible
+        _titler_data.layers[layer_index].visible = toggled_visible
+        self.layer_list.fill_data_model()
+
+        self.view_editor.edit_area.queue_draw()
+        
     def _center_h_pressed(self):
         # calculate top left x pos for centering
         w, h = _titler_data.active_layer.pango_layout.pixel_size
@@ -817,12 +845,15 @@ class PangoTextLayout:
 
 class TextLayerListView(gtk.VBox):
 
-    def __init__(self, selection_changed_cb):
+    def __init__(self, selection_changed_cb, layer_visible_toggled_cb):
         gtk.VBox.__init__(self)
         self.layer_icon = gtk.gdk.pixbuf_new_from_file(respaths.IMAGE_PATH + "text_layer.png")
-        
+        self.eye_icon = gtk.gdk.pixbuf_new_from_file(respaths.IMAGE_PATH + "eye.png")
+
+        self.layer_visible_toggled_cb = layer_visible_toggled_cb
+
        # Datamodel: str
-        self.storemodel = gtk.ListStore(gtk.gdk.Pixbuf, str)
+        self.storemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, gtk.gdk.Pixbuf)
  
         # Scroll container
         self.scroll = gtk.ScrolledWindow()
@@ -833,22 +864,30 @@ class TextLayerListView(gtk.VBox):
         self.treeview = gtk.TreeView(self.storemodel)
         self.treeview.set_property("rules_hint", True)
         self.treeview.set_headers_visible(False)
+        self.treeview.connect("button-press-event", self.button_press)
+        
         tree_sel = self.treeview.get_selection()
         tree_sel.set_mode(gtk.SELECTION_SINGLE)
         tree_sel.connect("changed", selection_changed_cb)
-            
-        # Column view
-        self.text_col_1 = gtk.TreeViewColumn("text1")
-        self.icon_col_1 = gtk.TreeViewColumn("Icon")
-        
+
         # Cell renderers
         self.text_rend_1 = gtk.CellRendererText()
         self.text_rend_1.set_property("ellipsize", pango.ELLIPSIZE_END)
         self.text_rend_1.set_property("font", "Sans Bold 10")
         self.text_rend_1.set_fixed_height_from_font(1)
+
         self.icon_rend_1 = gtk.CellRendererPixbuf()
         self.icon_rend_1.props.xpad = 6
         self.icon_rend_1.set_fixed_size(40, 40)
+
+        self.icon_rend_2 = gtk.CellRendererPixbuf()
+        self.icon_rend_2.props.xpad = 2
+        self.icon_rend_2.set_fixed_size(20, 40)
+
+        # Column view
+        self.icon_col_1 = gtk.TreeViewColumn("layer_icon")
+        self.text_col_1 = gtk.TreeViewColumn("layer_text")
+        self.icon_col_2 = gtk.TreeViewColumn("eye_icon")
         
         # Build column views
         self.icon_col_1.set_expand(False)
@@ -863,15 +902,27 @@ class TextLayerListView(gtk.VBox):
         self.text_col_1.pack_start(self.text_rend_1)
         self.text_col_1.add_attribute(self.text_rend_1, "text", 1)
 
+        self.icon_col_2.set_expand(False)
+        self.icon_col_2.set_spacing(5)
+        self.icon_col_2.pack_start(self.icon_rend_2)
+        self.icon_col_2.add_attribute(self.icon_rend_2, 'pixbuf', 2)
+
         # Add column views to view
         self.treeview.append_column(self.icon_col_1)
         self.treeview.append_column(self.text_col_1)
+        self.treeview.append_column(self.icon_col_2)
 
         # Build widget graph and display
         self.scroll.add(self.treeview)
         self.pack_start(self.scroll)
         self.scroll.show_all()
 
+    def button_press(self, tree_view, event):
+        if self.icon_col_1.get_width() + self.text_col_1.get_width() < event.x:
+            path = self.treeview.get_path_at_pos(int(event.x), int(event.y))
+            if path != None:
+                self.layer_visible_toggled_cb(max(path[0]))
+
     def get_selected_row(self):
         model, rows = self.treeview.get_selection().get_selected_rows()
         # When listening to "changed" this gets called too often for our needs (namely when user types),
@@ -889,7 +940,11 @@ class TextLayerListView(gtk.VBox):
         """
         self.storemodel.clear()
         for layer in _titler_data.layers:
-            row_data = [self.layer_icon, layer.text]
+            if layer.visible:
+                visible_icon = self.eye_icon
+            else:
+                visible_icon = None 
+            row_data = [self.layer_icon, layer.text, visible_icon]
             self.storemodel.append(row_data)
         
         self.scroll.queue_draw()
