@@ -21,8 +21,8 @@
 """
 Module contains GUI update routines.
 """
+
 import gtk
-import threading
 
 import appconsts
 import clipeffectseditor
@@ -35,10 +35,8 @@ from editorstate import PLAYER
 from editorstate import PROJECT
 from editorstate import timeline_visible
 import editorpersistance
-import mlt
-import panels
+import monitorevent
 import utils
-import render
 import respaths
 import tlinewidgets
 
@@ -69,7 +67,7 @@ stop_icon = None
 stop_trim_icon = None
 
 # Callback func to set default editmode, set from outside of the module.
-set_default_edit_mode_callback = None
+set_clip_edit_mode_callback = None
 
 # Timeline position is saved when clip is displayed
 saved_timeline_pos = -1
@@ -98,22 +96,19 @@ def load_icons():
 
 
 # --------------------------------- player
-def set_playing_configuration():
-    gui.editor_window.play_b.set_sensitive(False)
-    gui.editor_window.stop_b.set_sensitive(True)
-    
-def set_stopped_configuration():
-    gui.editor_window.play_b.set_sensitive(True)
-    gui.editor_window.stop_b.set_sensitive(False)
-
 def refresh_player():
     # First event is initial window displayed event.
-    # Last closing event needs to blocked by setting this flag
+    # Last closing event needs to be blocked by setting this flag
     # before calling window hide
     global player_refresh_enabled
     if not player_refresh_enabled:
         player_refresh_enabled = True
         return
+    # Refreshing while rendering overwrites file on disk and loses 
+    # previous rendered data. 
+    if PLAYER().is_rendering:
+        return
+
     PLAYER().refresh()
 
 # --------------------------------- window 
@@ -267,7 +262,7 @@ def mouse_scroll_zoom(event):
             zoom_in()
         else:
             zoom_out()
-        
+
 def maybe_autocenter():
     if timeline_visible():
         if editorpersistance.prefs.auto_center_on_play_stop == True:
@@ -275,7 +270,7 @@ def maybe_autocenter():
 
 
 # ----------------------------------------- monitor
-def display_clip_in_monitor(reset_saved_frames=True):
+def display_clip_in_monitor(clip_monitor_currently_active=False):
     """
     Sets mltplayer producer to be video file clip and updates GUI.
     """
@@ -285,37 +280,51 @@ def display_clip_in_monitor(reset_saved_frames=True):
 
     # Opening clip exits trim modes
     if not editorstate.current_is_move_mode():
-        set_default_edit_mode_callback()
+        set_clip_edit_mode_callback()
 
     gui.clip_editor_b.set_sensitive(True)
     editorstate._timeline_displayed = False
 
     # Save timeline pos if so directed.
-    if reset_saved_frames == True:
+    if clip_monitor_currently_active == False:
         global saved_timeline_pos
         saved_timeline_pos = PLAYER().current_frame()
-        tlinewidgets.shadow_frame = saved_timeline_pos
+        editorstate.tline_shadow_frame = saved_timeline_pos
+
+    # If we're already displaying monitor clip we stop consumer 
+    # to supress timeline flashing between monitor clips
+    if clip_monitor_currently_active == False:
+        editorstate.PLAYER().consumer.stop()
 
     # Clear old clip
     current_sequence().clear_hidden_track()
     
     # Create and display clip on hidden track
-    if MONITOR_MEDIA_FILE().type != appconsts.PATTERN_PRODUCER:
+    if MONITOR_MEDIA_FILE().type == appconsts.PATTERN_PRODUCER or MONITOR_MEDIA_FILE().type == appconsts.IMAGE_SEQUENCE:
+        # pattern producer or image sequence
+        clip_producer = current_sequence().display_monitor_clip(None, MONITOR_MEDIA_FILE())
+    else:
         # File producers
         clip_producer = current_sequence().display_monitor_clip(MONITOR_MEDIA_FILE().path)
-    else:
-        # pattern producer
-        clip_producer = current_sequence().display_monitor_clip(None, MONITOR_MEDIA_FILE())
 
+    # Timeline flash does not happen if we start consumer after monitor clip is 
+    # already on sequence
+    if clip_monitor_currently_active == False:
+        editorstate.PLAYER().consumer.start()
+        
+    # IMAGE_SEQUENCE files always returns 15000 for get_length from mlt so we have to monkeypatch that method to get correct results
+    if MONITOR_MEDIA_FILE().type == appconsts.IMAGE_SEQUENCE:
+        clip_producer.get_length = lambda : MONITOR_MEDIA_FILE().length
+    
     clip_producer.mark_in = MONITOR_MEDIA_FILE().mark_in
     clip_producer.mark_out = MONITOR_MEDIA_FILE().mark_out
     
-    # Give IMAGE and PATTERN_PRODUCER media types defalt mark in and mark out if not already set.
-    # This makes them reasonably short and trimmable in both directions by default
+    # Give IMAGE and PATTERN_PRODUCER media types default mark in and mark out if not already set.
+    # This makes them reasonably short and trimmable in both directions.
     if clip_producer.media_type == appconsts.IMAGE or clip_producer.media_type == appconsts.PATTERN_PRODUCER:
         if  clip_producer.mark_in == -1 and clip_producer.mark_out == -1:
             center_frame = clip_producer.get_length() / 2
-            default_length_half = 75 # TODO: we configurable and always equaling 6s
+            default_length_half = 75
             mark_in = center_frame - default_length_half
             mark_out = center_frame + default_length_half - 1
             clip_producer.mark_in = mark_in
@@ -323,20 +332,31 @@ def display_clip_in_monitor(reset_saved_frames=True):
             MONITOR_MEDIA_FILE().mark_in = mark_in
             MONITOR_MEDIA_FILE().mark_out = mark_out
 
-    # Display clip name
-    gui.editor_window.monitor_source.set_text(MONITOR_MEDIA_FILE().name)
+    display_monitor_clip_name()
 
     # Display frame, marks and pos
     gui.pos_bar.update_display_from_producer(clip_producer)
     PLAYER().seek_frame(0)
     display_marks_tc()
     
+    gui.pos_bar.widget.grab_focus()
+    gui.media_list_view.widget.queue_draw()
+    
     if editorpersistance.prefs.auto_play_in_clip_monitor == True:
         PLAYER().start_playback()
-        set_playing_configuration()
     
     repaint_tline()
 
+def display_monitor_clip_name():
+    # Display clip name
+    if  MONITOR_MEDIA_FILE().mark_in != -1 and MONITOR_MEDIA_FILE().mark_out != -1:
+        clip_length = utils.get_tc_string(MONITOR_MEDIA_FILE().mark_out - MONITOR_MEDIA_FILE().mark_in + 1) #+1 out incl.
+        range_text = " / ][ " + str(clip_length)
+    else:
+        range_text = ""
+
+    gui.editor_window.monitor_source.set_text(MONITOR_MEDIA_FILE().name + range_text)
+
 def display_sequence_in_monitor():
     """
     Sets mltplayer producer to be current sequence
@@ -363,8 +383,12 @@ def display_sequence_in_monitor():
     saved_timeline_pos = -1
 
     # Display sequence name
-    name = current_sequence().name
-    gui.editor_window.monitor_source.set_text(name)
+    name = editorstate.current_sequence().name
+    profile_desc = editorstate.current_sequence().profile.description()
+    if editorpersistance.prefs.show_sequence_profile:
+        gui.editor_window.monitor_source.set_text(name + " / " + profile_desc)
+    else:
+        gui.editor_window.monitor_source.set_text(name)
     
     # Display marks and pos 
     gui.pos_bar.update_display_from_producer(PLAYER().producer)
@@ -372,6 +396,15 @@ def display_sequence_in_monitor():
 
     repaint_tline()
 
+def switch_monitor_display():
+    monitorevent.stop_pressed()
+    if editorstate.MONITOR_MEDIA_FILE() == None:
+        return
+    if editorstate._timeline_displayed == True:
+        gui.editor_window.clip_editor_b.set_active(True)
+    else:
+        gui.editor_window.sequence_editor_b.set_active(True)
+
 def display_tline_cut_frame(track, index):
     """
     Displays sequence frame at cut
@@ -407,7 +440,7 @@ def set_and_display_monitor_media_file(media_file):
     # If we're displaying sequence we do programmatical click on "Clip" button 
     # to display clip via it's signal listener. 
     if gui.editor_window.clip_editor_b.get_active() == True:
-        display_clip_in_monitor(reset_saved_frames = False)
+        display_clip_in_monitor(clip_monitor_currently_active = True)
     else:
         gui.editor_window.clip_editor_b.set_active(True)
 
@@ -424,57 +457,20 @@ def update_frame_displayers(frame):
     norm_pos = frame / float(producer_length) 
     gui.pos_bar.set_normalized_pos(norm_pos)
 
-    # Update monitor tc
-    gui.tc.set_frame(frame)
-    
-    # Repaint timeline widgets if sequence displayed.
-    if timeline_visible():
-        gui.tline_scale.widget.queue_draw()
-        gui.tline_canvas.widget.queue_draw()
-        gui.big_tc.widget.queue_draw()
-        clipeffectseditor.display_kfeditors_tline_frame(frame)
-        compositeeditor.display_kfeditors_tline_frame(frame)
+    gui.tline_scale.widget.queue_draw()
+    gui.tline_canvas.widget.queue_draw()
+    gui.big_tc.widget.queue_draw()
+    clipeffectseditor.display_kfeditors_tline_frame(frame)
+    compositeeditor.display_kfeditors_tline_frame(frame)
 
 def update_kf_editor():
     clipeffectseditor.update_kfeditors_positions()
 
 # ----------------------------------------- marks
 def display_marks_tc():
-    if timeline_visible():
-        mark_in = PLAYER().producer.mark_in
-        mark_out = PLAYER().producer.mark_out
-    else:
-        mark_in = current_sequence().monitor_clip.mark_in
-        mark_out = current_sequence().monitor_clip.mark_out
-        
-    if mark_in != -1:
-        mark_in_tc = utils.get_tc_string(mark_in) 
-        gui.mark_in_display.set_text(mark_in_tc)
-    else:
-        gui.mark_in_display.set_text("--:--:--:--")
-    
-    if mark_out != -1:
-        mark_out_tc = utils.get_tc_string(mark_out) 
-        gui.mark_out_display.set_text(mark_out_tc)
-    else:
-        gui.mark_out_display.set_text("--:--:--:--")
-        
-    if ((mark_out != -1) and (mark_in != -1)):
-        length = mark_out - mark_in + 1 # + 1 == out inclusive
-        l_tc = utils.get_tc_string(length)
-        gui.length_display.set_text(l_tc)
-    else:
-        if timeline_visible():
-            gui.length_display.set_text("--:--:--:--")
-        else: # length for a clip is always displayed
-            if mark_out == -1 and mark_in == -1:
-                l = current_sequence().monitor_clip.get_length()
-            elif mark_out == -1:
-                l = current_sequence().monitor_clip.get_length() - mark_in
-            else:
-                l = mark_out
-            l_tc = utils.get_tc_string(l)
-            gui.length_display.set_text(l_tc)
+    if not timeline_visible():
+        display_monitor_clip_name()
+
 
 # ----------------------------------------------- clip editors
 def clip_removed_during_edit(clip):
@@ -485,80 +481,40 @@ def clear_clip_from_editors(clip):
         clipeffectseditor.clear_clip()
 
 # ----------------------------------------- edit modes
-def set_mode_button_active(mode):
-    gui.mode_buttons[mode].set_active(True)
-
 def set_trim_mode_gui():
     """
     Called when user selects trim mode
     """
     display_sequence_in_monitor()
 
-    _set_move_mode_buttons_enabled(False)
-    gui.pos_bar.disabled = True
-    gui.editor_window.play_b.set_image(play_loop_icon)
-    gui.editor_window.stop_b.set_image(stop_trim_icon)
-    gui.editor_window.next_b.set_image(next_trim_icon)
-    gui.editor_window.prev_b.set_image(prev_trim_icon)
-    
-    gui.editmenu.set_sensitive(False)
-
 def set_move_mode_gui():
     """
     Called when user selects move mode
     """
     display_sequence_in_monitor()
-    
-    _set_move_mode_buttons_enabled(True)
-    gui.pos_bar.disabled = False
-    gui.editor_window.play_b.set_image(play_icon)
-    gui.editor_window.stop_b.set_image(stop_icon)
-    gui.editor_window.next_b.set_image(next_icon)
-    gui.editor_window.prev_b.set_image(prev_icon)
 
-    gui.editmenu.set_sensitive(True)
-    
-def _set_move_mode_buttons_enabled(enabled):
-    """
-    Sets buttons that are only used in move modes enabled/disabled
-    """
-    # Timeline edit buttons
-    gui.editor_window.zoom_in_b.set_sensitive(enabled)
-    gui.editor_window.zoom_out_b.set_sensitive(enabled)
-    gui.editor_window.lift_b.set_sensitive(enabled)
-    gui.editor_window.cut_b.set_sensitive(enabled)
-    gui.editor_window.splice_out_b.set_sensitive(enabled)
-    gui.editor_window.overwrite_b.set_sensitive(enabled)
-    gui.editor_window.insert_b.set_sensitive(enabled)
-    gui.editor_window.append_b.set_sensitive(enabled)
-    gui.editor_window.resync_b.set_sensitive(enabled)
-    gui.editor_window.zoom_length_b.set_sensitive(enabled)
-
-    # Monitor buttons
-    gui.editor_window.rew_b.set_sensitive(enabled) 
-    gui.editor_window.ff_b.set_sensitive(enabled) 
-    gui.editor_window.mark_in_b.set_sensitive(enabled) 
-    gui.editor_window.mark_out_b.set_sensitive(enabled) 
-    gui.editor_window.marks_clear_b.set_sensitive(enabled)  
-    gui.editor_window.to_mark_in_b.set_sensitive(enabled)
-    gui.editor_window.to_mark_out_b .set_sensitive(enabled)
-    
-def set_next_prev_enabled(enabled):
-    """
-    Sets buttons enbled when doing trim playback.
-    """
-    gui.editor_window.next_b.set_sensitive(enabled)  
-    gui.editor_window.prev_b.set_sensitive(enabled)  
-
-def update_project_info(project):
-    for child in gui.project_info_vbox.get_children():
-        gui.project_info_vbox.remove(child)
-    name_panel = panels.get_project_name_panel(project.name)
-    profile_info = panels.get_profile_info_panel(project.profile)
-    gui.project_info_vbox.pack_start(name_panel, False, True, 0)
-    gui.project_info_vbox.pack_start(profile_info, False, True, 0)
-    gui.project_info_vbox.show_all()
+def set_transition_render_edit_menu_items_sensitive(range_start, range_end):
+    if not editorstate.current_is_move_mode():
+        return
 
+    ui = gui.editor_window.uimanager
+    render_transition = ui.get_widget('/MenuBar/EditMenu/AddTransition')
+    render_fade = ui.get_widget('/MenuBar/EditMenu/AddFade')
+    if range_start == -1:
+        render_transition.set_sensitive(False)
+        render_fade.set_sensitive(False)
+    elif range_start == range_end:
+        render_transition.set_sensitive(False)
+        render_fade.set_sensitive(True)
+    elif range_start == range_end - 1:
+        render_transition.set_sensitive(True)
+        render_fade.set_sensitive(False)
+    else:
+        render_transition.set_sensitive(False)
+        render_fade.set_sensitive(False)
 
+# ------------------------------------------------ notebook
+def switch_notebook_panel(index):
+    gui.middle_notebook.set_current_page(index)
 
 
